﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.Serialization;
using System.ServiceModel;
using System.Text;
using System.Data;
using DATAVideoClub;

namespace WCFVideoClub
{
    public class Service1 : IService1
    {
        //Datacontext que utilitzarem per accedir a la base de dades.
        private VideoClubDataClassesDataContext db;

        //Mètode per obrir la connexió amb la BD
        private void ConnexioDataContext()
        {
            try
            {
                //Creem una nova instancia del datacontext.
                db = new VideoClubDataClassesDataContext();
                //Obrim la connexió.
                db.Connection.Open();
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }

        // Mètode per tancar la connexió amb la BD
        private void DesconnexioDataContext()
        {
            try
            {
                //Tanquem la connexió.
                db.Connection.Close();
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }
         
        //Mètode que rep com a paràmetre un nom d'usuari i una contrasenya per comprovar si
        //existeixen a la BD. Si existeix retorna l'ID d'usuari, altrament, 0.
        public int AccesUsuari(String nomUsuari, int password)
        {
            try
            {
                //Obrim la connexió.
                ConnexioDataContext();

                //Consulta per trobar si existeix un usuari amb nomUsuari i password.
                var usuari = (from u in db.USUARI
                              where u.DNI.StartsWith(nomUsuari) && u.CONTRASENYA == password
                             select u).FirstOrDefault();

                //Tanquem la connexió
                DesconnexioDataContext();

                if (usuari != null)
                {
                    return usuari.ID_USUARI;
                }
                else
                {
                    return 0;
                }
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }

        //Mètode per a obtenir el nom complet d'un usuari a partir del seu Id
        public String ObtenirNomUsuari(int idUsuari)
        {
            try
            {
                //Obrim la connexió.
                ConnexioDataContext();

                //Consulta per trobar si existeix un usuari amb nomUsuari i password.
                var usuari = (from u in db.USUARI
                              where u.ID_USUARI == idUsuari
                              select u).FirstOrDefault();

                //Tanquem la connexió
                DesconnexioDataContext();

                if (usuari != null)
                {
                    return usuari.NOM + " " + usuari.COGNOM1 + " " + usuari.COGNOM2;
                }
                else
                {
                    return "";
                }
            }
            catch (NullReferenceException)
            {
               throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }

        //Mètode per obtenir un llistat amb tots els gèneres actius
        public List<GENERE> ObtenirGeneres(int Filtre)
        {
            try
            {
                List<GENERE> LlistatGeneres = new List<GENERE>();

                ConnexioDataContext();

                switch (Filtre)
                {
                    case 0:
                        var query = (from g in db.GENERE
                                     orderby g.GENERE1
                                     select g).ToList();
                        LlistatGeneres = query;
                        break;
                    case 1:
                        var query1 = (from g in db.GENERE
                                      where g.ACTIU == 1
                                      orderby g.GENERE1
                                      select g).ToList();
                        LlistatGeneres = query1;
                        break;
                    case 2:
                        var query2 = (from g in db.GENERE
                                      where g.ACTIU == 0
                                      orderby g.GENERE1
                                      select g).ToList();
                        LlistatGeneres = query2;
                        break;
                }

                DesconnexioDataContext();

                return LlistatGeneres;
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }

        //Mètode per obtenir la descripció d'un gènere a partir del seu Identificador
        public String ObtenirGenere(int idGenere)
        {
            try
            {                
                ConnexioDataContext();

                var genere = (from g in db.GENERE
                              where g.ID_GENERE == idGenere
                              select g).FirstOrDefault();

                DesconnexioDataContext();

                if (genere != null) 
                { 
                    return genere.GENERE1;
                }else{
                    if (idGenere == 0) //Si l'identificador = 0 correspon a tots els gèneres
                    {
                        return "Tots els gèneres";
                    }else{
                        return "";
                    }                    
                }
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }

        //Mètode per obenir un string amb els gèneres als quals pertany una pel.lícula (idPellicula)
        public String ObtenirGeneresPellicula(int idPellicula)
        {
            String StrGeneres = "";

            try
            {
                ConnexioDataContext();

                var generes = (from g in db.GENERE_PELLICULA 
                              where g.ID_PELLICULA == idPellicula                               
                              select g).ToList();

                DesconnexioDataContext();

                int i = 0;

                foreach (GENERE_PELLICULA genere in generes)
                {
                    StrGeneres = StrGeneres + ObtenirGenere(genere.ID_GENERE).ToString();
                    i++;
                    if (i < generes.Count) { StrGeneres = StrGeneres + ", "; }
                }

                return StrGeneres;
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }

        //Mètode per obtenir un llistat amb tots els formats actius que hi ha a la BD
        public List<FORMAT> ObtenirFormats(int Filtre)
        {
            try
            {
                List<FORMAT> LlistatFormats = new List<FORMAT>();

                ConnexioDataContext();

                switch (Filtre)
                {
                    case 0:
                        var query = (from f in db.FORMAT                                       
                                       orderby f.FORMAT1
                                       select f).ToList();
                        LlistatFormats = query;
                        break;
                    case 1:
                        var query1 = (from f in db.FORMAT
                                     where f.ACTIU == 1
                                     orderby f.FORMAT1
                                     select f).ToList();
                        LlistatFormats = query1;
                        break;
                    case 2:
                        var query2 = (from f in db.FORMAT
                                     where f.ACTIU == 0
                                     orderby f.FORMAT1
                                     select f).ToList();
                        LlistatFormats = query2;
                        break;
                }

                DesconnexioDataContext();

                return LlistatFormats;
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }

        //Mètode per obtenir la descripció d'un format a partir del seu identificador
        public String ObtenirFormat(int idFormat)
        {
            try
            {
                ConnexioDataContext();

                var format = (from f in db.FORMAT
                              where f.ID_FORMAT == idFormat
                              select f).FirstOrDefault();

                DesconnexioDataContext();

                if (format != null)
                {
                    return format.FORMAT1;
                }
                else
                {
                    return "";
                }
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }

        //Mètode per obtenir el llistat de pel.lícules de lloguer en format (idFormat) i que pertanyen al gènere (idGenere)
        public List<PELLICULA> ObtenirPelliculesLloguer(int idFormat, int idGenere)
        {
            try
            {
                ConnexioDataContext();

                List<PELLICULA> pelicules = new List<PELLICULA>();
                
                if (idGenere == 0)
                {
                    var query = (from pellicula in db.PELLICULA
                                from unitat_pellicula in db.UNITAT_PELLICULA
                                from format_unitat_pellicula in db.FORMAT_UNITAT_PELLICULA
                                where pellicula.ID_PELLICULA == unitat_pellicula.ID_PELLICULA &&
                                unitat_pellicula.ID_UNITAT == format_unitat_pellicula.ID_UNITAT &&
                                unitat_pellicula.LLOGUER == 1 &&
                                pellicula.ACTIVA == 1 &&
                                unitat_pellicula.ACTIVA == 1 &&
                                format_unitat_pellicula.ID_FORMAT == idFormat 
                                select new
                                {
                                pellicula.ID_PELLICULA,
                                pellicula.TITOL,
                                pellicula.DIRECTOR,
                                pellicula.PAIS,
                                pellicula.ANY,
                                pellicula.DATA_ESTRENA,
                                pellicula.DURACIO,
                                pellicula.QUALIFICACIO,
                                pellicula.REPARTIMENT,
                                pellicula.ARGUMENT,
                                pellicula.CARATULA_FRONTAL,
                                pellicula.CARATULA_POSTERIOR,
                                pellicula.VISIONAT_PREVI,
                                pellicula.ACTIVA,
                                pellicula.DATA_ALTA,
                                pellicula.DATA_BAIXA
                                }).Distinct();

                    pelicules.Clear();

                    foreach (var r in query)
                    {
                        PELLICULA pelicula = new PELLICULA();

                        pelicula.ID_PELLICULA = r.ID_PELLICULA;
                        pelicula.TITOL = r.TITOL;
                        pelicula.DIRECTOR = r.DIRECTOR;
                        pelicula.PAIS = r.PAIS;
                        pelicula.ANY = r.ANY;
                        pelicula.DATA_ESTRENA = r.DATA_ESTRENA;
                        pelicula.DURACIO = r.DURACIO;
                        pelicula.QUALIFICACIO = r.QUALIFICACIO;
                        pelicula.REPARTIMENT = r.REPARTIMENT;
                        pelicula.ARGUMENT = r.ARGUMENT;
                        pelicula.CARATULA_FRONTAL = r.CARATULA_FRONTAL;
                        pelicula.CARATULA_POSTERIOR = r.CARATULA_POSTERIOR;
                        pelicula.VISIONAT_PREVI = r.VISIONAT_PREVI;
                        pelicula.ACTIVA = r.ACTIVA;
                        pelicula.DATA_ALTA = r.DATA_ALTA;
                        pelicula.DATA_BAIXA = r.DATA_BAIXA;

                        pelicules.Add(pelicula);
                    }
                }else{
                    var query = (from pellicula in db.PELLICULA
                                from unitat_pellicula in db.UNITAT_PELLICULA
                                from format_unitat_pellicula in db.FORMAT_UNITAT_PELLICULA
                                from genere_pellicula in db.GENERE_PELLICULA
                                where pellicula.ID_PELLICULA == unitat_pellicula.ID_PELLICULA &&
                                pellicula.ID_PELLICULA == genere_pellicula.ID_PELLICULA &&
                                unitat_pellicula.ID_UNITAT == format_unitat_pellicula.ID_UNITAT &&
                                unitat_pellicula.LLOGUER == 1 &&
                                pellicula.ACTIVA == 1 &&
                                unitat_pellicula.ACTIVA == 1 &&
                                format_unitat_pellicula.ID_FORMAT == idFormat &&
                                genere_pellicula.ID_GENERE == idGenere
                                select new
                                {
                                pellicula.ID_PELLICULA,
                                pellicula.TITOL,
                                pellicula.DIRECTOR,
                                pellicula.PAIS,
                                pellicula.ANY,
                                pellicula.DATA_ESTRENA,
                                pellicula.DURACIO,
                                pellicula.QUALIFICACIO,
                                pellicula.REPARTIMENT,
                                pellicula.ARGUMENT,
                                pellicula.CARATULA_FRONTAL,
                                pellicula.CARATULA_POSTERIOR,
                                pellicula.VISIONAT_PREVI,
                                pellicula.ACTIVA,
                                pellicula.DATA_ALTA,
                                pellicula.DATA_BAIXA
                                }).Distinct();

                    pelicules.Clear();

                    foreach (var r in query)
                    {
                        PELLICULA pelicula = new PELLICULA();

                        pelicula.ID_PELLICULA = r.ID_PELLICULA;
                        pelicula.TITOL = r.TITOL;
                        pelicula.DIRECTOR = r.DIRECTOR;
                        pelicula.PAIS = r.PAIS;
                        pelicula.ANY = r.ANY;
                        pelicula.DATA_ESTRENA = r.DATA_ESTRENA;
                        pelicula.DURACIO = r.DURACIO;
                        pelicula.QUALIFICACIO = r.QUALIFICACIO;
                        pelicula.REPARTIMENT = r.REPARTIMENT;
                        pelicula.ARGUMENT = r.ARGUMENT;
                        pelicula.CARATULA_FRONTAL = r.CARATULA_FRONTAL;
                        pelicula.CARATULA_POSTERIOR = r.CARATULA_POSTERIOR;
                        pelicula.VISIONAT_PREVI = r.VISIONAT_PREVI;
                        pelicula.ACTIVA = r.ACTIVA;
                        pelicula.DATA_ALTA = r.DATA_ALTA;
                        pelicula.DATA_BAIXA = r.DATA_BAIXA;

                        pelicules.Add(pelicula);
                    }
                }
                
                DesconnexioDataContext();
                
                return pelicules;
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }

        //Mètode per obtenir el llistat de pel.lícules de venda en format (idFormat) i que pertanyen al gènere (idGenere)
        public List<PELLICULA> ObtenirPelliculesVenda(int idFormat, int idGenere)
        {
            try
            {
                ConnexioDataContext();

                List<PELLICULA> pelicules = new List<PELLICULA>();

                if (idGenere == 0)
                {
                    var query = (from pellicula in db.PELLICULA
                                 from unitat_pellicula in db.UNITAT_PELLICULA
                                 from format_unitat_pellicula in db.FORMAT_UNITAT_PELLICULA
                                 where pellicula.ID_PELLICULA == unitat_pellicula.ID_PELLICULA &&
                                 unitat_pellicula.ID_UNITAT == format_unitat_pellicula.ID_UNITAT &&
                                 unitat_pellicula.VENDA == 1 &&
                                 pellicula.ACTIVA == 1 &&
                                 unitat_pellicula.ACTIVA == 1 &&
                                 format_unitat_pellicula.ID_FORMAT == idFormat
                                 select new
                                 {
                                     pellicula.ID_PELLICULA,
                                     pellicula.TITOL,
                                     pellicula.DIRECTOR,
                                     pellicula.PAIS,
                                     pellicula.ANY,
                                     pellicula.DATA_ESTRENA,
                                     pellicula.DURACIO,
                                     pellicula.QUALIFICACIO,
                                     pellicula.REPARTIMENT,
                                     pellicula.ARGUMENT,
                                     pellicula.CARATULA_FRONTAL,
                                     pellicula.CARATULA_POSTERIOR,
                                     pellicula.VISIONAT_PREVI,
                                     pellicula.ACTIVA,
                                     pellicula.DATA_ALTA,
                                     pellicula.DATA_BAIXA
                                 }).Distinct();

                    pelicules.Clear();

                    foreach (var r in query)
                    {
                        PELLICULA pelicula = new PELLICULA();

                        pelicula.ID_PELLICULA = r.ID_PELLICULA;
                        pelicula.TITOL = r.TITOL;
                        pelicula.DIRECTOR = r.DIRECTOR;
                        pelicula.PAIS = r.PAIS;
                        pelicula.ANY = r.ANY;
                        pelicula.DATA_ESTRENA = r.DATA_ESTRENA;
                        pelicula.DURACIO = r.DURACIO;
                        pelicula.QUALIFICACIO = r.QUALIFICACIO;
                        pelicula.REPARTIMENT = r.REPARTIMENT;
                        pelicula.ARGUMENT = r.ARGUMENT;
                        pelicula.CARATULA_FRONTAL = r.CARATULA_FRONTAL;
                        pelicula.CARATULA_POSTERIOR = r.CARATULA_POSTERIOR;
                        pelicula.VISIONAT_PREVI = r.VISIONAT_PREVI;
                        pelicula.ACTIVA = r.ACTIVA;
                        pelicula.DATA_ALTA = r.DATA_ALTA;
                        pelicula.DATA_BAIXA = r.DATA_BAIXA;

                        pelicules.Add(pelicula);
                    }
                }
                else
                {
                    var query = (from pellicula in db.PELLICULA
                                 from unitat_pellicula in db.UNITAT_PELLICULA
                                 from format_unitat_pellicula in db.FORMAT_UNITAT_PELLICULA
                                 from genere_pellicula in db.GENERE_PELLICULA
                                 where pellicula.ID_PELLICULA == unitat_pellicula.ID_PELLICULA &&
                                 pellicula.ID_PELLICULA == genere_pellicula.ID_PELLICULA &&
                                 unitat_pellicula.ID_UNITAT == format_unitat_pellicula.ID_UNITAT &&
                                 unitat_pellicula.VENDA == 1 &&
                                 pellicula.ACTIVA == 1 &&
                                 unitat_pellicula.ACTIVA == 1 &&
                                 format_unitat_pellicula.ID_FORMAT == idFormat &&
                                 genere_pellicula.ID_GENERE == idGenere
                                 select new
                                 {
                                     pellicula.ID_PELLICULA,
                                     pellicula.TITOL,
                                     pellicula.DIRECTOR,
                                     pellicula.PAIS,
                                     pellicula.ANY,
                                     pellicula.DATA_ESTRENA,
                                     pellicula.DURACIO,
                                     pellicula.QUALIFICACIO,
                                     pellicula.REPARTIMENT,
                                     pellicula.ARGUMENT,
                                     pellicula.CARATULA_FRONTAL,
                                     pellicula.CARATULA_POSTERIOR,
                                     pellicula.VISIONAT_PREVI,
                                     pellicula.ACTIVA,
                                     pellicula.DATA_ALTA,
                                     pellicula.DATA_BAIXA
                                 }).Distinct();

                    pelicules.Clear();

                    foreach (var r in query)
                    {
                        PELLICULA pelicula = new PELLICULA();

                        pelicula.ID_PELLICULA = r.ID_PELLICULA;
                        pelicula.TITOL = r.TITOL;
                        pelicula.DIRECTOR = r.DIRECTOR;
                        pelicula.PAIS = r.PAIS;
                        pelicula.ANY = r.ANY;
                        pelicula.DATA_ESTRENA = r.DATA_ESTRENA;
                        pelicula.DURACIO = r.DURACIO;
                        pelicula.QUALIFICACIO = r.QUALIFICACIO;
                        pelicula.REPARTIMENT = r.REPARTIMENT;
                        pelicula.ARGUMENT = r.ARGUMENT;
                        pelicula.CARATULA_FRONTAL = r.CARATULA_FRONTAL;
                        pelicula.CARATULA_POSTERIOR = r.CARATULA_POSTERIOR;
                        pelicula.VISIONAT_PREVI = r.VISIONAT_PREVI;
                        pelicula.ACTIVA = r.ACTIVA;
                        pelicula.DATA_ALTA = r.DATA_ALTA;
                        pelicula.DATA_BAIXA = r.DATA_BAIXA;

                        pelicules.Add(pelicula);
                    }
                }

                DesconnexioDataContext();

                return pelicules;
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }

        //Mètode per a obtenir l'estat d'una pel.lícula (Disponible, Reservada o Llogada)
        public String ObtenirEstatPellicula(int idPellicula, int idFormat)
        {
            String Estat = "A";

            try
            {
                ConnexioDataContext();

                var query = from unitat_pellicula in db.UNITAT_PELLICULA
                            from format_unitat_pellicula in db.FORMAT_UNITAT_PELLICULA
                            where unitat_pellicula.ID_UNITAT == format_unitat_pellicula.ID_UNITAT &&
                            unitat_pellicula.ID_PELLICULA == idPellicula &&
                            format_unitat_pellicula.ID_FORMAT == idFormat
                            orderby unitat_pellicula.ESTAT
                            select new
                            {
                                unitat_pellicula.ESTAT
                            };

                foreach (var r in query)
                {
                    if (r.ESTAT.ToString() == "D" && Estat != "D")
                    {
                        Estat = r.ESTAT.ToString();
                    }

                    if (r.ESTAT.ToString() == "R" && Estat != "D")
                    {
                        Estat = r.ESTAT.ToString();
                    }
                }
                    
                DesconnexioDataContext();

                if (query != null)
                {
                    return Estat;
                }
                else
                {
                    return "";
                }
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }

        //Mètode per obtenir l'identificador d'una copia disponible d'una pel.lícula
        public int ObtenirCopiaDisponiblePellicula(int idPellicula, int idFormat)
        {
            try
            {
                ConnexioDataContext();

                var query = from unitat_pellicula in db.UNITAT_PELLICULA
                            from format_unitat_pellicula in db.FORMAT_UNITAT_PELLICULA
                            where unitat_pellicula.ID_UNITAT == format_unitat_pellicula.ID_UNITAT &&
                            unitat_pellicula.ID_PELLICULA == idPellicula &&
                            format_unitat_pellicula.ID_FORMAT == idFormat && 
                            unitat_pellicula.ESTAT.Equals("D") 
                            orderby unitat_pellicula.ESTAT
                            select new
                            {
                                unitat_pellicula.ID_UNITAT
                            };
                               
                DesconnexioDataContext();

                if (query != null)
                {
                    return query.FirstOrDefault().ID_UNITAT;
                }
                else
                {
                    return 0;
                }
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }

        //Mètode per obtenir un llistat amb tots els usuaris que hi ha registrats a la BD
        public List<USUARI> ObtenirUsuaris(int Filtre)
        {
            try
            {
                List<USUARI> LlistatUsuaris = new List<USUARI>();

                ConnexioDataContext();

                switch (Filtre)
                { 
                    case 0:
                        var query = (from u in db.USUARI
                                       where u.IS_ADMINISTRADOR == 0
                                       orderby u.NOM
                                       select u).ToList();
                        LlistatUsuaris = query;
                        break;
                    case 1:
                        var query1 = (from u in db.USUARI
                                       where u.IS_ADMINISTRADOR == 0 && u.ACTIU == 1
                                       orderby u.NOM
                                       select u).ToList();
                        LlistatUsuaris = query1;
                        break;
                    case 2:
                        var query2 = (from u in db.USUARI
                                       where u.IS_ADMINISTRADOR == 0 && u.ACTIU == 0
                                       orderby u.NOM
                                       select u).ToList();
                        LlistatUsuaris = query2;
                        break;
                }

                DesconnexioDataContext();

                return LlistatUsuaris;
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }

        //Mètode per a obtenir el preu d'una pel.lícula (idPellicula)
        public Double ObtenirPreuArticle(int IdPellicula)
        {
            try
            {
                ConnexioDataContext();

                var unitats = (from u in db.UNITAT_PELLICULA 
                               where u.VENDA==1 && u.ID_PELLICULA == IdPellicula                                
                               select u).FirstOrDefault();

                DesconnexioDataContext();

                return (Double)unitats.PREU_VENDA;
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }

        //Mètode per realitzar la reserva (lloguer/compra) d'un article
        public Boolean RealitzarReserva(int IdUsuari, int IdUnitatArticle, int TipusReserva)
        {
            try
            {
                ConnexioDataContext();

                //Donem d'alta la reserva
                RESERVA r = new RESERVA();
                r.ID_USUARI = IdUsuari;
                r.ID_UNITAT = IdUnitatArticle;
                r.ACTIVA = 1;
                r.DATA_RESERVA = DateTime.Now;
                r.TIPUS_RESERVA = TipusReserva;
                db.RESERVA.InsertOnSubmit(r);
                //Canviem l'estat de l'article a "R" (Reservada)
                UNITAT_PELLICULA UnitatArticle = db.UNITAT_PELLICULA.Single(up => up.ID_UNITAT == IdUnitatArticle);
                UnitatArticle.ESTAT = 'R';                
                db.SubmitChanges();

                DesconnexioDataContext();

                return true;
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }

        public List<VISTA_RESERVES_ACTIVES> ObtenirLlistatReserves(int IdUsuari)
        {
            try
            {
                List<VISTA_RESERVES_ACTIVES> LlistatReserves = new List<VISTA_RESERVES_ACTIVES>();
                LlistatReserves.Clear();

                ConnexioDataContext();

                if (IdUsuari == 0)
                {
                    var query = (from vista_reserves_actives in db.VISTA_RESERVES_ACTIVES
                    select vista_reserves_actives).ToList();

                    LlistatReserves = query;
                }else{    
                    var query = (from vista_reserves_actives in db.VISTA_RESERVES_ACTIVES
                                 where vista_reserves_actives.ID_USUARI == IdUsuari 
                    select vista_reserves_actives).ToList();

                    LlistatReserves = query;
                }

                DesconnexioDataContext();

                return LlistatReserves;
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }

        public List<VISTA_LLOGUERS_ACTIUS> ObtenirLlistatLloguers(int IdUsuari)
        {
            List<VISTA_LLOGUERS_ACTIUS> LlistatLloguers = new List<VISTA_LLOGUERS_ACTIUS>();
            LlistatLloguers.Clear();

            try
            {
                ConnexioDataContext();

                if (IdUsuari == 0)
                {
                    var query = (from vista_lloguers_actius in db.VISTA_LLOGUERS_ACTIUS
                                 select vista_lloguers_actius).ToList();

                    LlistatLloguers = query;
                }
                else
                {
                    var query = (from vista_lloguers_actius in db.VISTA_LLOGUERS_ACTIUS
                                 where vista_lloguers_actius.ID_USUARI == IdUsuari
                                 select vista_lloguers_actius).ToList();
                  
                    LlistatLloguers = query;
                }

                DesconnexioDataContext();

                return LlistatLloguers;
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }

        public USUARI BuscarUsuari(String DNI)
        {
            try
            {
                ConnexioDataContext();

                var usuari = (from u in db.USUARI
                                where u.DNI.StartsWith(DNI)
                                select u).FirstOrDefault();

                DesconnexioDataContext();

                if (usuari != null)
                {
                    return usuari;
                }
                else
                {
                    return null;
                }
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }

        public Boolean AcceptarReserva(int IdReserva)
        {
            try
            {
                ConnexioDataContext();

                RESERVA reserva = db.RESERVA.Single(r => r.ID_RESERVA == IdReserva);

                if (reserva != null)
                {
                    switch (reserva.TIPUS_RESERVA)
                    {
                        case 1:  //Lloguer
                            LLOGUER l = new LLOGUER();
                            l.ID_USUARI = reserva.ID_USUARI;
                            l.ID_UNITAT = reserva.ID_UNITAT;
                            l.DATA_LLOGUER = DateTime.Now;
                            db.LLOGUER.InsertOnSubmit(l);
                            //Canviem l'estat de l'article a "A" (Alquilada)
                            UNITAT_PELLICULA UnitatArticleLlogat = db.UNITAT_PELLICULA.Single(up => up.ID_UNITAT == reserva.ID_UNITAT);
                            UnitatArticleLlogat.ESTAT = 'A';
                            //Donem de baixa la reserva
                            reserva.ACTIVA = 0;                            
                            db.SubmitChanges();
                            break;
                        case 2:  //Venda
                            VENDA v = new VENDA();
                            v.ID_USUARI = reserva.ID_USUARI;
                            v.ID_UNITAT = reserva.ID_UNITAT;
                            v.DATA_VENDA = DateTime.Now;
                            db.VENDA.InsertOnSubmit(v);
                            //Canviem l'estat de l'article a "V" (Venut) i el donem de baixa
                            UNITAT_PELLICULA UnitatArticleVenut = db.UNITAT_PELLICULA.Single(up => up.ID_UNITAT == reserva.ID_UNITAT);
                            UnitatArticleVenut.ESTAT = 'V';
                            UnitatArticleVenut.ACTIVA = 0;
                            UnitatArticleVenut.DATA_BAIXA = DateTime.Now;
                            //Donem de baixa la reserva
                            reserva.ACTIVA = 0;                            
                            db.SubmitChanges();
                            break;
                    }
                    DesconnexioDataContext();    
                    return true;                                    
                }
                else
                {
                    DesconnexioDataContext();
                    return false;
                }                
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }

        public Boolean CancelarReserva(int IdReserva)
        {
            try
            {
                ConnexioDataContext();

                RESERVA reserva = db.RESERVA.Single(r => r.ID_RESERVA == IdReserva);

                if (reserva != null)
                {
                    reserva.ACTIVA = 0;                    
                    db.SubmitChanges();
                    DesconnexioDataContext();
                    return true;
                }else{
                    DesconnexioDataContext();
                    return false;
                }
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }

        public Boolean TornarArticleLlogat(int IdLloguer, Double PreuLloguer)
        {
            try
            {
                ConnexioDataContext();

                //Recuperem el lloguer
                LLOGUER lloguer = db.LLOGUER.Single(ll => ll.ID_LLOGUER == IdLloguer);

                if (lloguer != null)
                {
                    //Registrem la devolució
                    DEVOLUCIO d = new DEVOLUCIO();
                    d.DATA_DEVOLUCIO = DateTime.Now;
                    d.PREU_LLOGUER = PreuLloguer;
                    db.DEVOLUCIO.InsertOnSubmit(d);
                    db.SubmitChanges();
                    
                    //Donem de baixa el lloguer, és a dir, li associem una devolució
                    lloguer.ID_DEVOLUCIO = d.ID_DEVOLUCIO;
                    
                    //Recuperem l'article i li canviem l'estat a 'D' (Disponible
                    UNITAT_PELLICULA unitatpellicula = db.UNITAT_PELLICULA.Single(up => up.ID_UNITAT == lloguer.ID_UNITAT);
                    unitatpellicula.ESTAT = 'D';

                    db.SubmitChanges();
                    DesconnexioDataContext();
                    return true;
                }
                else
                {
                    DesconnexioDataContext();
                    return false;
                }                    
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }

        public Double CalculaPreuLloguer(DateTime dataLloguer, DateTime dataDevolucio)
        {
            try
            {
                ConnexioDataContext();

                Double preuLloguer = 0;

                //Recuperem els paràmetres de l'aplicació
                PARAMETRES_APLICACIO appParam = new PARAMETRES_APLICACIO();
                appParam = ObtenirParametresAplicacio();

                System.TimeSpan TSDiferencia = dataDevolucio.Subtract(dataLloguer);

                if (TSDiferencia.Days > 0)
                {
                    preuLloguer = TSDiferencia.Days * 24 * appParam.PREU_HORA_LLOGUER;
                }

                if (TSDiferencia.Hours > 0)
                {
                    preuLloguer = preuLloguer + (TSDiferencia.Hours * appParam.PREU_HORA_LLOGUER);
                }

                if (TSDiferencia.Minutes > 0 && preuLloguer == 0)
                {
                    preuLloguer = appParam.PREU_MINIM_LLOGUER;
                }

                DesconnexioDataContext();                    
                return preuLloguer;
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }            
        }

        public PARAMETRES_APLICACIO ObtenirParametresAplicacio()
        {
            try
            {
                ConnexioDataContext();

                var parametres = (from p in db.PARAMETRES_APLICACIO                          
                              select p).FirstOrDefault();

                DesconnexioDataContext();

                if (parametres != null)
                {
                    return parametres;
                }else{
                    return null;
                }
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }                  
        }

        //Mètode per obtenir l'usuari Administrador
        public USUARI ObtenirUsuariAdministrador()
        {
            try
            {
                ConnexioDataContext();

                var admin = (from u in db.USUARI
                               where u.IS_ADMINISTRADOR == 1
                               orderby u.NOM
                               select u).FirstOrDefault();

                DesconnexioDataContext();

                return admin;
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }

        public void CanviaContrasenyaAdministrador(int novaContrasenya)
        {
            try
            {
                USUARI Admin = ObtenirUsuariAdministrador();

                if (Admin != null)
                {
                    ConnexioDataContext();

                    Admin.CONTRASENYA = novaContrasenya;
                    db.SubmitChanges();

                    DesconnexioDataContext();                
                }
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }

        //Mètode per obtenir tots els articles actius que hi ha registrats a la BD
        public List<VISTA_ARTICLES_ACTIUS> ObtenirLlistatTotsArticles()
        {
            try
            {
                ConnexioDataContext();

                var query = (from vista_articles_actius in db.VISTA_ARTICLES_ACTIUS
                             select vista_articles_actius).ToList();
                
                DesconnexioDataContext();

                return query;
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }

        //Mètode per obtenir tots els articles de lloguer
        public List<VISTA_ARTICLES_ACTIUS_LLOGUER> ObtenirLlistatArticlesLloguer()
        {
            try
            {
                ConnexioDataContext();

                var query = (from vista_articles_actius_lloguer in db.VISTA_ARTICLES_ACTIUS_LLOGUER
                             select vista_articles_actius_lloguer).ToList();

                DesconnexioDataContext();

                return query;
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }

        //Mètode per obtenir tots els articles de venda
        public List<VISTA_ARTICLES_ACTIUS_VENDA> ObtenirLlistatArticlesVenda()
        {
            try
            {
                ConnexioDataContext();

                var query = (from vista_articles_actius_venda in db.VISTA_ARTICLES_ACTIUS_VENDA
                             select vista_articles_actius_venda).ToList();

                DesconnexioDataContext();

                return query;
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }

        //Mètode per comprovar si existeix un usuari
        public Boolean ExisteixUsuari(String dni)
        {
            try
            {
                ConnexioDataContext();

                var usuari = (from usuaris in db.USUARI
                              where usuaris.DNI == dni 
                              select usuaris).FirstOrDefault();

                DesconnexioDataContext();

                if (usuari != null)
                {
                    return true;
                }else{
                    return false;
                }
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }

        //Mètode per a donar d'alta nous usuaris
        public void AltaUsuari(USUARI NewUsuari)
        {
            try
            {
                ConnexioDataContext();

                USUARI u = new USUARI();
                u.DNI = NewUsuari.DNI.ToUpper();
                u.NOM = NewUsuari.NOM;
                u.COGNOM1 = NewUsuari.COGNOM1;
                u.COGNOM2 = NewUsuari.COGNOM2;
                u.CONTRASENYA = NewUsuari.CONTRASENYA;
                u.ADREÇA = NewUsuari.ADREÇA;
                u.POBLACIO = NewUsuari.POBLACIO;
                u.CP = NewUsuari.CP;
                u.PROVINCIA = NewUsuari.PROVINCIA;
                u.TELEFON1 = NewUsuari.TELEFON1;
                u.TELEFON2 = NewUsuari.TELEFON2;
                u.IS_ADMINISTRADOR = 0;
                u.ACTIU = 1;
                u.DATA_ALTA = DateTime.Now;                
                db.USUARI.InsertOnSubmit(u);
                db.SubmitChanges();

                DesconnexioDataContext();
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }

        //Mètode per a donar de baixa un usuari
        public void BaixaUsuari(int IdUsuari)
        {
            try
            {
                ConnexioDataContext();

                USUARI usuari = db.USUARI.Single(u => u.ID_USUARI == IdUsuari);
                usuari.ACTIU = 0;
                usuari.DATA_BAIXA = DateTime.Now;
                db.SubmitChanges();
                DesconnexioDataContext();
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }

        //Mètode per a modificar les dades d'un usuari
        public void ModificarUsuari(USUARI NewUsuari)
        {
            try
            {
                ConnexioDataContext();

                USUARI usuari = db.USUARI.Single(u => u.ID_USUARI == NewUsuari.ID_USUARI);
                usuari.DNI = NewUsuari.DNI.ToUpper();
                usuari.NOM = NewUsuari.NOM;
                usuari.COGNOM1 = NewUsuari.COGNOM1;
                usuari.COGNOM2 = NewUsuari.COGNOM2;
                usuari.CONTRASENYA = NewUsuari.CONTRASENYA;
                usuari.ADREÇA = NewUsuari.ADREÇA;
                usuari.POBLACIO = NewUsuari.POBLACIO;
                usuari.CP = NewUsuari.CP;
                usuari.PROVINCIA = NewUsuari.PROVINCIA;
                usuari.TELEFON1 = NewUsuari.TELEFON1;
                usuari.TELEFON2 = NewUsuari.TELEFON2;                
                db.SubmitChanges();

                DesconnexioDataContext();
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }

        //Mètode per a donar d'alta noves pel.lícules
        public int AltaPellicula(PELLICULA NewPellicula, List<GENERE> LlistaGeneres)
        {
            try
            {
                ConnexioDataContext();

                //Donem d'alta la pel.lícula
                PELLICULA p = new PELLICULA();
                p.TITOL = NewPellicula.TITOL;
                p.DIRECTOR = NewPellicula.DIRECTOR;
                p.PAIS = NewPellicula.PAIS;
                p.ANY = NewPellicula.ANY;
                p.DATA_ESTRENA = NewPellicula.DATA_ESTRENA;
                p.DURACIO = NewPellicula.DURACIO;
                p.QUALIFICACIO = NewPellicula.QUALIFICACIO;
                p.REPARTIMENT = NewPellicula.REPARTIMENT;
                p.ARGUMENT = NewPellicula.ARGUMENT;
                p.ACTIVA = 1;
                p.DATA_ALTA = DateTime.Now;
                p.CARATULA_FRONTAL = NewPellicula.CARATULA_FRONTAL;
                p.CARATULA_POSTERIOR = NewPellicula.CARATULA_POSTERIOR;
                p.VISIONAT_PREVI = NewPellicula.VISIONAT_PREVI;
                db.PELLICULA.InsertOnSubmit(p);
                db.SubmitChanges();

                //Donem d'alta els gèneres als que pertany la pel.lícula
                for (int i = 0; i < LlistaGeneres.Count; i++)
                {
                    GENERE_PELLICULA g = new GENERE_PELLICULA();
                    g.ID_GENERE = LlistaGeneres[i].ID_GENERE;
                    g.ID_PELLICULA = p.ID_PELLICULA;
                    db.GENERE_PELLICULA.InsertOnSubmit(g);
                    db.SubmitChanges();
                }
                
                DesconnexioDataContext();
                return p.ID_PELLICULA;
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }

        //Mètode per a donar de baixa una pel.lícula
        public void BaixaPellicula(int IdPellicula)
        {
            try
            {
                ConnexioDataContext();

                PELLICULA pellicula = db.PELLICULA.Single(p => p.ID_PELLICULA == IdPellicula);
                pellicula.ACTIVA = 0;
                pellicula.DATA_BAIXA = DateTime.Now;
                db.SubmitChanges();
                DesconnexioDataContext();
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }

        //Mètode per a modificar les dades d'una pellicula
        public void ModificarPellicula(PELLICULA NewPellicula)
        {
            try
            {
                ConnexioDataContext();

                PELLICULA pellicula = db.PELLICULA.Single(p => p.ID_PELLICULA == NewPellicula.ID_PELLICULA);
                pellicula.TITOL = NewPellicula.TITOL;
                pellicula.DIRECTOR = NewPellicula.DIRECTOR;
                pellicula.PAIS = NewPellicula.PAIS;
                pellicula.ANY = NewPellicula.ANY;
                pellicula.DATA_ESTRENA = NewPellicula.DATA_ESTRENA;
                pellicula.DURACIO = NewPellicula.DURACIO;
                pellicula.QUALIFICACIO = NewPellicula.QUALIFICACIO;
                pellicula.REPARTIMENT = NewPellicula.REPARTIMENT;
                pellicula.ARGUMENT = NewPellicula.ARGUMENT;
                pellicula.CARATULA_FRONTAL = NewPellicula.CARATULA_FRONTAL;
                pellicula.CARATULA_POSTERIOR = NewPellicula.CARATULA_POSTERIOR;
                pellicula.VISIONAT_PREVI = NewPellicula.VISIONAT_PREVI;
                db.SubmitChanges();

                DesconnexioDataContext();
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }

        //Mètode per a obtenir el llistat de còpies d'una pel.lícula (IdPellicula)
        public List<UNITAT_PELLICULA> ObtenirLlistatCopiesPellicula(int IdPellicula)
        {
            try
            {
                ConnexioDataContext();

                var query = (from cp in db.UNITAT_PELLICULA
                             where cp.ID_PELLICULA == IdPellicula
                             select cp).ToList();
                
                DesconnexioDataContext();

                return query;
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }
                
        //Mètode per a donar d'alta una còpia de pel.lícula
        public int AltaCopiaPellicula(UNITAT_PELLICULA NewCopiaPellicula, int IdPellicula, int IdFormat)
        {
            try
            {
                ConnexioDataContext();

                //Donem d'alta la còpia
                UNITAT_PELLICULA up = new UNITAT_PELLICULA();
                up.ID_PELLICULA = IdPellicula;
                up.ESTAT = 'D';
                up.LLOGUER = NewCopiaPellicula.LLOGUER;
                up.VENDA = NewCopiaPellicula.VENDA;
                up.PREU_VENDA = NewCopiaPellicula.PREU_VENDA;
                up.ACTIVA = 1;
                up.DATA_ALTA = DateTime.Now;
                db.UNITAT_PELLICULA.InsertOnSubmit(up);
                db.SubmitChanges();

                //Donem d'alta el format de la còpia
                FORMAT_UNITAT_PELLICULA fp = new FORMAT_UNITAT_PELLICULA();
                fp.ID_UNITAT = up.ID_UNITAT;
                fp.ID_FORMAT = IdFormat;
                db.FORMAT_UNITAT_PELLICULA.InsertOnSubmit(fp);
                db.SubmitChanges();

                DesconnexioDataContext();
                return up.ID_UNITAT;
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }

        //Mètode per a donar de baixa una còpia de pel.lícula
        public void BaixaCopiaPellicula(int IdCopiaPellicula)
        {
            try
            {
                ConnexioDataContext();

                UNITAT_PELLICULA copia = db.UNITAT_PELLICULA.Single(up => up.ID_UNITAT == IdCopiaPellicula);
                copia.ACTIVA = 0;
                copia.DATA_BAIXA = DateTime.Now;
                db.SubmitChanges();
                DesconnexioDataContext();
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }

        //Mètode per a modficar les dades d'una còpia de pel.lícula
        public void ModificarCopiaPellicula(UNITAT_PELLICULA NewCopiaPellicula, int IdFormat)
        {
            try
            {
                ConnexioDataContext();

                UNITAT_PELLICULA copia = db.UNITAT_PELLICULA.Single(up => up.ID_UNITAT == NewCopiaPellicula.ID_UNITAT);
                copia.LLOGUER = NewCopiaPellicula.LLOGUER;
                copia.VENDA = NewCopiaPellicula.VENDA;
                copia.PREU_VENDA = NewCopiaPellicula.PREU_VENDA;                
                db.SubmitChanges();

                FORMAT_UNITAT_PELLICULA fp = db.FORMAT_UNITAT_PELLICULA.Single(up => up.ID_UNITAT == NewCopiaPellicula.ID_UNITAT);                
                fp.ID_FORMAT = IdFormat;
                db.SubmitChanges();

                DesconnexioDataContext();
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }
        
        //Mètode per esborrar els gèneres d'una pel.lícula
        public void EsborrarGeneresPellicula(int IdPellicula)
        { 
            try
            {
                ConnexioDataContext();

                //Donem de baixa els gèneres actuals de la pel.lícula
                var generes = from gp in db.GENERE_PELLICULA
                               where gp.ID_PELLICULA == IdPellicula
                               select gp;

                foreach (var g in generes)
                {
                    db.GENERE_PELLICULA.DeleteOnSubmit(g);                    
                }

                db.SubmitChanges();

                DesconnexioDataContext();
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }

        //Mètode per afegir un gènere a una pel.lícula
        public void AfegirGenerePellicula(int IdPellicula, int IdGenere)
        {
            try
            {
                ConnexioDataContext();

                GENERE_PELLICULA g = new GENERE_PELLICULA();
                g.ID_GENERE = IdGenere;
                g.ID_PELLICULA = IdPellicula;

                db.GENERE_PELLICULA.InsertOnSubmit(g);               
                db.SubmitChanges();

                DesconnexioDataContext();
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }

        //Mètode per comprovar si ja existeix un títol de pel.lícula a la BD
        public Boolean ExisteixPellicula(String TitolPellicula)
        {
            try
            {
                ConnexioDataContext();

                var pellicula = (from p in db.PELLICULA
                                 where p.TITOL.ToUpper() == TitolPellicula.ToUpper()
                              select p).FirstOrDefault();

                DesconnexioDataContext();

                if (pellicula != null)
                {
                    return true;
                }else{
                    return false;
                }
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }

        //Mètode que retorna una llista amb tots els gèneres als que pertany una pel.lícula
        public List<GENERE> ObtenirLlistaGeneresPellicula(int IdPellicula)
        {
            try
            {
                ConnexioDataContext();

                var query = from genere_pellicula in db.GENERE_PELLICULA
                            where genere_pellicula.ID_PELLICULA == IdPellicula
                            select new{
                                genere_pellicula.ID_GENERE,
                                genere_pellicula.ID_PELLICULA
                            };

                List<GENERE> llistaGeneres = new List<GENERE>();
                llistaGeneres.Clear();

                foreach (var g in query)
                {
                    GENERE genere = new GENERE();
                    genere.ID_GENERE = g.ID_GENERE;
                    genere.GENERE1 = ObtenirGenere(g.ID_GENERE);
                    llistaGeneres.Add(genere);
                }

                DesconnexioDataContext();

                return llistaGeneres;            
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }

        //Mètode que retorna una llista amb tots els gèneres als que NO pertany una pel.lícula
        public List<GENERE> ObtenirLlistaGeneresNOPellicula(int IdPellicula)
        {
            try
            {
                ConnexioDataContext();

                var generesActius = (from genere in db.GENERE
                             where genere.ACTIU == 1
                             select new
                             {
                                 ID_GENERE = (int)genere.ID_GENERE,
                                 genere.GENERE1
                             });

                var generesPellicula = (from genere_pellicula in db.GENERE_PELLICULA
                                        where genere_pellicula.ID_PELLICULA == IdPellicula
                                        select new
                                        {
                                            ID_GENERE = (int)genere_pellicula.ID_GENERE,                                 
                                        });

                List<GENERE> llistaGeneres = new List<GENERE>();
                llistaGeneres.Clear();

                Boolean addGenere;

                foreach (var ga in generesActius)
                {
                    GENERE genere = new GENERE();
                    genere.ID_GENERE = ga.ID_GENERE;
                    genere.GENERE1 = ga.GENERE1;

                    addGenere = true;

                    foreach (var gp in generesPellicula)
                    {
                        if (genere.ID_GENERE == gp.ID_GENERE)
                        {
                            addGenere = false;
                            break;
                        }
                    }

                    if (addGenere == true) { llistaGeneres.Add(genere); }
                }

                DesconnexioDataContext();

                return llistaGeneres;
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }

        //Mètode per guardar els paràmetres de configuració
        public void GuardarConfiguracio(PARAMETRES_APLICACIO NewParametres)
        {
            try
            {
                ConnexioDataContext();

                var parametres = (from parametres_aplicacio in db.PARAMETRES_APLICACIO                                  
                                  select parametres_aplicacio).FirstOrDefault();

                if (parametres != null)
                {
                    parametres.RAO_SOCIAL = NewParametres.RAO_SOCIAL;
                    parametres.CIF = NewParametres.CIF;
                    parametres.ADREÇA = NewParametres.ADREÇA;
                    parametres.EMAIL = NewParametres.EMAIL;
                    parametres.TELEFON = NewParametres.TELEFON;
                    parametres.FAX = NewParametres.FAX;
                    parametres.WEB = NewParametres.WEB;
                    parametres.MAX_ARTICLES_LLOGATS_USUARI = NewParametres.MAX_ARTICLES_LLOGATS_USUARI;
                    parametres.MAX_HORES_LLOGUER = NewParametres.MAX_HORES_LLOGUER;
                    parametres.MAX_MINUTS_RECOLLIR_RESERVA = NewParametres.MAX_MINUTS_RECOLLIR_RESERVA;
                    parametres.MAX_MINUTS_UTILITZACIO_TERMINAL = NewParametres.MAX_MINUTS_UTILITZACIO_TERMINAL;
                    parametres.PREU_HORA_LLOGUER = NewParametres.PREU_HORA_LLOGUER;
                    parametres.PREU_MINIM_LLOGUER = NewParametres.PREU_MINIM_LLOGUER;
                    parametres.RUTA_IMATGES = NewParametres.RUTA_IMATGES;
                    parametres.RUTA_VIDEOS = NewParametres.RUTA_VIDEOS;
                    db.SubmitChanges();
                }else{
                    db.PARAMETRES_APLICACIO.InsertOnSubmit(NewParametres);
                    db.SubmitChanges();
                }

                DesconnexioDataContext();
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }

        //Mètode per a obtenir una pel.lícula a partir del seu ID
        public PELLICULA ObtenirPellicula(int IdPellicula)
        {
            try
            {
                ConnexioDataContext();

                var pellicula = (from p in db.PELLICULA
                                 where p.ID_PELLICULA == IdPellicula
                                 select p).FirstOrDefault();

                DesconnexioDataContext();

                if (pellicula != null)
                {
                    return pellicula;
                }else{
                    return null;
                }
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }

        //Mètode per a obtenir l'identificador del format d'un article a partir del seu id
        public int ObtenirFormatUnitat(int IdUnitat)
        {
            try
            {
                ConnexioDataContext();

                var format = (from fup in db.FORMAT_UNITAT_PELLICULA
                                 where fup.ID_UNITAT == IdUnitat
                                 select fup).FirstOrDefault();

                DesconnexioDataContext();

                if (format != null)
                {
                    return format.ID_FORMAT;
                }
                else
                {
                    return 0;
                }
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }

        //Mètode per comprovar si ja existeix un format a la BD
        public int ExisteixFormat(String DescripcioFormat)
        {
            try
            {
                ConnexioDataContext();

                var format = (from f in db.FORMAT
                              where f.FORMAT1.ToUpper() == DescripcioFormat.ToUpper()
                              select f).FirstOrDefault();

                DesconnexioDataContext();

                if (format != null)
                {
                    return format.ID_FORMAT;
                }else{
                    return 0;
                }
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }

        //Mètode per donar d'alta un nou format
        public int AltaFormat(String NewFormat)
        {
            try
            {
                ConnexioDataContext();

                FORMAT f = new FORMAT();
                f.FORMAT1 = NewFormat;
                f.ACTIU = 1;
                f.DATA_ALTA = DateTime.Now;
                db.FORMAT.InsertOnSubmit(f);
                db.SubmitChanges();

                DesconnexioDataContext();

                return f.ID_FORMAT;
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }

        //Mètode per a donar de baixa un format existent
        public void BaixaFormat(int IdFormat)
        {
            try
            {
                ConnexioDataContext();

                FORMAT format = db.FORMAT.Single(f => f.ID_FORMAT == IdFormat);
                format.ACTIU = 0;
                format.DATA_BAIXA = DateTime.Now;
                db.SubmitChanges();
                DesconnexioDataContext();
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }

        //Mètode per a activar un format existent
        public int ActivarFormat(int IdFormat)
        {
            try
            {
                ConnexioDataContext();

                FORMAT format = db.FORMAT.Single(f => f.ID_FORMAT == IdFormat);
                format.ACTIU = 1;
                format.DATA_BAIXA = null;
                db.SubmitChanges();
                DesconnexioDataContext();

                if (format != null)
                {
                    return format.ID_FORMAT;
                }else{
                    return 0;
                }

            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }

        //Mètode per comprovar si ja existeix un gènere a la BD
        public int ExisteixGenere(String DescripcioGenere)
        {
            try
            {
                ConnexioDataContext();

                var genere = (from g in db.GENERE
                              where g.GENERE1.ToUpper() == DescripcioGenere.ToUpper()
                              select g).FirstOrDefault();

                DesconnexioDataContext();

                if (genere != null)
                {
                    return genere.ID_GENERE;
                }else{
                    return 0;
                }
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }

        //Mètode per donar d'alta un nou gènere
        public int AltaGenere(String NewGenere)
        {
            try
            {
                ConnexioDataContext();

                GENERE g = new GENERE();
                g.GENERE1 = NewGenere;
                g.ACTIU = 1;
                g.DATA_ALTA = DateTime.Now;
                db.GENERE.InsertOnSubmit(g);
                db.SubmitChanges();

                DesconnexioDataContext();

                return g.ID_GENERE;
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }

        //Mètode per a donar de baixa un gènere existent
        public void BaixaGenere(int IdGenere)
        {
            try
            {
                ConnexioDataContext();

                GENERE genere = db.GENERE.Single(g => g.ID_GENERE == IdGenere);
                genere.ACTIU = 0;
                genere.DATA_BAIXA = DateTime.Now;
                db.SubmitChanges();
                DesconnexioDataContext();
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }

        //Mètode per a activar un gènere existent
        public int ActivarGenere(int IdGenere)
        {
            try
            {
                ConnexioDataContext();

                GENERE genere = db.GENERE.Single(g => g.ID_GENERE == IdGenere);
                genere.ACTIU = 1;
                genere.DATA_BAIXA = null;
                db.SubmitChanges();
                DesconnexioDataContext();

                if (genere !=null)
                {
                    return genere.ID_GENERE;
                }else{
                    return 0;
                }
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }

        //Mètode per a obtnir el llistat de lloguers que estan fora del termini de devolució
        public List<VISTA_LLOGUERS_FORA_TERMINI_DEVOLUCIO> ObtenirLlistaLloguersForaTerminiDevolucio()
        {
            try
            {
                List<VISTA_LLOGUERS_FORA_TERMINI_DEVOLUCIO> LlistatLloguersForaTermini = new List<VISTA_LLOGUERS_FORA_TERMINI_DEVOLUCIO>();
                LlistatLloguersForaTermini.Clear();

                ConnexioDataContext();

                var query = (from vista_lloguers_fora_termini_devolucio in db.VISTA_LLOGUERS_FORA_TERMINI_DEVOLUCIO
                             select vista_lloguers_fora_termini_devolucio).ToList();

                LlistatLloguersForaTermini = query;

                DesconnexioDataContext();

                return LlistatLloguersForaTermini;
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }

        //Mètode per a obtenir un llistat ordenat amb els articles de més a menys productius (econòmicament parlant)
        public List<VISTA_RENDIMENT_ECONOMIC_ARTICLES> ObtenirLlistaRendimentEconomicArticles()
        {
            try
            {
                List<VISTA_RENDIMENT_ECONOMIC_ARTICLES> LlistatArticles = new List<VISTA_RENDIMENT_ECONOMIC_ARTICLES>();
                LlistatArticles.Clear();

                ConnexioDataContext();

                var query = (from vista_rendiment_economic_articles in db.VISTA_RENDIMENT_ECONOMIC_ARTICLES
                             select vista_rendiment_economic_articles).ToList();

                LlistatArticles = query;

                DesconnexioDataContext();

                return LlistatArticles;
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }

        //Mètode per a obtenir un llistat de novetats (pel.lícules en que la data d'estrena és inferior a 6 mesos)
        public List<VISTA_NOVETATS> ObtenirLlistaNovetats()
        {
            try
            {
                List<VISTA_NOVETATS> LlistatNovetats = new List<VISTA_NOVETATS>();
                LlistatNovetats.Clear();

                ConnexioDataContext();

                var query = (from novetats in db.VISTA_NOVETATS                            
                             select novetats).ToList();

                LlistatNovetats = query;

                DesconnexioDataContext();

                return LlistatNovetats;
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }

        //Mètode per guardar temporalment les copies d'una pel.lícula i així facilitar la seva gestió
        public void GuardarTemporalCopiesPellicula(int IdPellicula)
        {
            try
            {
                ConnexioDataContext();

                BuidarTemporalCopiesPellicula();

                var query = (from copies in db.UNITAT_PELLICULA
                             where copies.ID_PELLICULA == IdPellicula && copies.ACTIVA == 1
                             select copies).ToList();

                foreach (var c in query)
                {
                    TMP_COPIES tmpC = new TMP_COPIES();
                    tmpC.ID_PELLICULA = c.ID_PELLICULA;
                    tmpC.ID_UNITAT = c.ID_UNITAT;
                    tmpC.ID_FORMAT = ObtenirFormatUnitat(c.ID_UNITAT);
                    tmpC.FORMAT = ObtenirFormat(tmpC.ID_FORMAT);
                    tmpC.LLOGUER = c.LLOGUER;
                    tmpC.VENDA = c.LLOGUER;
                    tmpC.PREU_VENDA = c.PREU_VENDA;
                    tmpC.BAIXA = 0;
                    db.TMP_COPIES.InsertOnSubmit(tmpC);
                    db.SubmitChanges();
                }

                DesconnexioDataContext();
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }

        //Mètode per buidar el temporal de còpies d'una pel.lícula
        public void BuidarTemporalCopiesPellicula()
        {
            try
            {
                ConnexioDataContext();

                var tmpC = from tc in db.TMP_COPIES                     
                           select tc;

                db.TMP_COPIES.DeleteAllOnSubmit(tmpC);
                db.SubmitChanges();

                DesconnexioDataContext();
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }

        //Mètode per a obtenir un llistat amb totes les còpies d'una pel.lícula del temporal
        public List<TMP_COPIES> ObtenirLlistatTemporalCopiesPellicula(Boolean TotesCopies)
        {
            try
            {
                ConnexioDataContext();

                if (TotesCopies == true)
                {
                    var query1 = (from cp in db.TMP_COPIES
                                  select cp).ToList();

                    DesconnexioDataContext();
                    return query1;
                }else{
                    var query2 = (from cp in db.TMP_COPIES
                                  where cp.BAIXA == 0
                                  select cp).ToList();

                    DesconnexioDataContext();
                    return query2;
                }
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }

        //Mètode per afegir una copia a la taula temporal
        public void AfegirCopiaPelliculaATemporal(TMP_COPIES copia)
        {
            try
            {
                ConnexioDataContext();

                TMP_COPIES c = copia;                
                db.TMP_COPIES.InsertOnSubmit(c);
                db.SubmitChanges();

                DesconnexioDataContext();
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }

        //Mètode per esborrar una còpia del temporal
        public void EsborrarCopiaPelliculaTemporal(int IdTemporal)
        {
            try
            {
                ConnexioDataContext();

                var query = (from cp in db.TMP_COPIES
                            where cp.ID_TMP == IdTemporal 
                            select cp).FirstOrDefault();

                if (query != null)
                {
                    query.BAIXA = 1;                    
                    db.SubmitChanges();
                }

                DesconnexioDataContext();
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }

        //Mètode per modificar les dades d'una còpia del temporal
        public void ModificarCopiaPelliculaTemporal(int IdTemporal, TMP_COPIES copia)
        {
            try
            {
                ConnexioDataContext();

                TMP_COPIES cp = db.TMP_COPIES.Single(c => c.ID_TMP == IdTemporal);
                cp.ID_FORMAT = copia.ID_FORMAT;
                cp.FORMAT = copia.FORMAT;
                cp.LLOGUER = copia.LLOGUER;
                cp.VENDA = copia.VENDA;
                cp.PREU_VENDA = copia.PREU_VENDA;
                db.SubmitChanges();

                DesconnexioDataContext();
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }

        //Mètode que retorna un Char corresponent a l'estat d'una pel.lícula
        public String ObtenirEstatCopiaPellicula(int IdCopia) 
        {
            try
            {
                ConnexioDataContext();
                
                var query = (from copia in db.UNITAT_PELLICULA 
                             where copia.ID_UNITAT == IdCopia
                             select copia).FirstOrDefault();

                DesconnexioDataContext();

                if (query != null)
                {
                    return query.ESTAT.ToString();
                }else{
                    return "";
                }
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }
    }
}