﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;
using System.IO;

using WPFGestioClients.WCFVideoClubServiceReference;

namespace WPFGestioClients
{
    /// <summary>
    /// Lógica de interacción para wndDetallsArticle.xaml
    /// </summary>
    public partial class wndDetallsArticle : Window
    {
        int IdUsuari;
        int IdCatalog;
        int IdFormat;
        PELLICULA Pellicula;
        PARAMETRES_APLICACIO appParam = new PARAMETRES_APLICACIO();

        Service1Client client = new Service1Client();

        public wndDetallsArticle(int paramIdUsuari, PELLICULA paramPellicula, String paramEstatPellicula, int paramCataleg, int paramFormat)
        {
            InitializeComponent();
            IdUsuari = paramIdUsuari;
            Pellicula = paramPellicula;
            IdCatalog = paramCataleg;
            IdFormat = paramFormat;
            appParam = client.ObtenirParametresAplicacio();

            lblDetall.Text = lblDetall.Text + " " + Pellicula.TITOL.ToString().ToUpper();
            lblTitol.Content = Pellicula.TITOL.ToString();
            lblDirector.Content = Pellicula.DIRECTOR.ToString();
            lblPais.Content = Pellicula.PAIS.ToString();
            lblAny.Content = Pellicula.ANY.ToString();
            lblDataEstrena.Content = Pellicula.DATA_ESTRENA.ToString();
            lblDuracio.Content = Pellicula.DURACIO.ToString() + " minuts";
            lblGenere.Content = client.ObtenirGeneresPellicula(Pellicula.ID_PELLICULA).ToString();
            lblQualificacio.Content = Pellicula.QUALIFICACIO.ToString();
            lblRepartiment.Text = Pellicula.REPARTIMENT.ToString();
            lblArgument.Text = Pellicula.ARGUMENT.ToString();

            if (paramCataleg == 2) //Article en venda
            {
                lblPreu.Visibility = Visibility.Visible;
                txtPreu.Visibility = Visibility.Visible;
                txtPreu.Text = client.ObtenirPreuArticle(Pellicula.ID_PELLICULA).ToString() + "€";
            }

            if (Pellicula.CARATULA_FRONTAL.Length > 0) { imgFront.Source = GetImatge(appParam.RUTA_IMATGES + (char)92 + Pellicula.CARATULA_FRONTAL.ToString()); }
            if (Pellicula.CARATULA_POSTERIOR.Length > 0) { imgBack.Source = GetImatge(appParam.RUTA_IMATGES + (char)92 + Pellicula.CARATULA_POSTERIOR.ToString()); }
            
            if (Pellicula.VISIONAT_PREVI.Length > 0)
            {
                btnVisionatPrevi.IsEnabled = true;
                lblVisionatPrevi.Foreground = Brushes.Navy;
            }else{
                btnVisionatPrevi.IsEnabled = false;
                lblVisionatPrevi.Foreground = Brushes.LightGray;
            }

            if (paramEstatPellicula == "DISPONIBLE")
            {
                btnReservar.IsEnabled = true;
                lblReservar.Foreground = Brushes.Navy;
            }else{
                btnReservar.IsEnabled = false;
                lblReservar.Foreground = Brushes.LightGray;
            }
        }

        private void btnTornar_Click(object sender, RoutedEventArgs e)
        {            
            this.Close();
        }

        //Reservem l'article
        private void btnReservar_Click(object sender, RoutedEventArgs e)
        {
            int idArticle = client.ObtenirCopiaDisponiblePellicula(Pellicula.ID_PELLICULA, IdFormat);
            
            if (idArticle != 0)
            {
                btnReservar.IsEnabled = true;
                lblReservar.Foreground = Brushes.Navy;
               
                if (client.RealitzarReserva(IdUsuari, idArticle, IdCatalog) == true)
                {
                    MessageBox.Show("Reserva realitzada correctament", "Reserva realitzada", MessageBoxButton.OK, MessageBoxImage.Information);
                    this.Close();
                }else{
                    btnReservar.IsEnabled = false;
                    lblReservar.Foreground = Brushes.LightGray;

                    MessageBox.Show("No s'ha pogut completar la reserva. Segurament, aquest article ja no está disponible.", "Reserva cancel.lada", MessageBoxButton.OK, MessageBoxImage.Stop);
                }
            }
            else
            {
                btnReservar.IsEnabled = false;
                lblReservar.Foreground = Brushes.LightGray;

                MessageBox.Show("No s'ha pogut completar la reserva. Segurament, aquest article ja no está disponible.", "Reserva cancel.lada", MessageBoxButton.OK, MessageBoxImage.Stop);
            }            
        }

        //Accedim al reproductor per veure el visionat previ
        private void btnVisionatPrevi_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                this.Cursor = Cursors.Wait;
                wndMediaPlayer windowVideoPlayer = new wndMediaPlayer(Pellicula.VISIONAT_PREVI.ToString());
                windowVideoPlayer.Show();
                this.Cursor = Cursors.Arrow;
            }
            catch (NullReferenceException ex)
            {
                MessageBox.Show(ex.Message, "VideoClub: " + ex.Source, MessageBoxButton.OK, MessageBoxImage.Error);
            }
            catch (SystemException ex1)
            {
                MessageBox.Show(ex1.Message, "VideoClub: " + ex1.Source, MessageBoxButton.OK, MessageBoxImage.Error);
            }
            catch (Exception ex2)
            {
                MessageBox.Show(ex2.Message, "VideoClub: " + ex2.Source, MessageBoxButton.OK, MessageBoxImage.Error);
            }
        }

        private BitmapImage GetImatge(String RutaImatge)
        {
            try
            {
                const String ImgNoDisponibleFile = @"pack://application:,,,/WPFGestioClients;component/Images/ImatgeNoDisponible.jpg";
                String CurFile = "";

                if (RutaImatge != "") { CurFile = RutaImatge; }

                if (!File.Exists(CurFile)) { CurFile = ImgNoDisponibleFile; }
                BitmapImage logo = new BitmapImage();
                logo.BeginInit();
                logo.UriSource = new Uri(CurFile);
                logo.EndInit();
                return logo;
            }
            catch (NullReferenceException ex)
            {
                MessageBox.Show(ex.Message, "VideoClub: " + ex.Source, MessageBoxButton.OK, MessageBoxImage.Error);
                return null;
            }
            catch (SystemException ex1)
            {
                MessageBox.Show(ex1.Message, "VideoClub: " + ex1.Source, MessageBoxButton.OK, MessageBoxImage.Error);
                return null;
            }
            catch (Exception ex2)
            {
                MessageBox.Show(ex2.Message, "VideoClub: " + ex2.Source, MessageBoxButton.OK, MessageBoxImage.Error);
                return null;
            }
        }
    }
}