﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;
using System.Windows.Threading;
using WPFGestioClients.WCFVideoClubServiceReference;

namespace WPFGestioClients
{    
    public partial class wndMediaPlayer : Window
    {
        String NomFitxerVideo;
        TimeSpan Posicio;
        DispatcherTimer Timer = new DispatcherTimer();
        PARAMETRES_APLICACIO appParam = new PARAMETRES_APLICACIO();

        Service1Client client = new Service1Client();

        public wndMediaPlayer(String paramFitxerVideo)
        {
            appParam = client.ObtenirParametresAplicacio();
            NomFitxerVideo = appParam.RUTA_VIDEOS + (char)92 + paramFitxerVideo;
            
            InitializeComponent();

            //Configurem el temporitzador per poder veure l'estat de reproducció
            Timer.Interval = TimeSpan.FromMilliseconds(1000);
            Timer.Tick += new EventHandler(ticktock);
            Timer.Start();

            //Iniciem la reproducció
            PlayVideoOnInit();
        }

        //Mostrem la posició actual de reproducció en la barra de progrés
        void ticktock(object sender, EventArgs e)
        {
            pbProgres.Value = mediaElement1.Position.TotalSeconds;            
        }

        //Comencem la reproducció del video
        private void PlayVideoOnInit() {            
            String FileName = NomFitxerVideo;

            if (System.IO.File.Exists(FileName))
            {
                try
                {
                    mediaElement1.Source = new Uri(FileName);
                    pbVolum.Value = 5;
                    mediaElement1.Volume = (double)(pbVolum.Value / 10);
                    mediaElement1.Play();
                }
                catch (System.IO.FileNotFoundException err)
                {
                    MessageBox.Show(err.Message);
                    this.Close();
                }
                catch (Exception ex)
                {
                    MessageBox.Show(ex.Message);
                    this.Close();
                }
            }
            else
            {
                MessageBox.Show("Fitxer no localitzat.");
                this.Close();
            }            
        }
        
        //Aturem la reproducció quan el video acava
        private void mediaElement1_MediaEnded(object sender, RoutedEventArgs e)
        {
            mediaElement1.Stop();
            btnPause.Visibility = Visibility.Hidden;
            btnPlay.Visibility = Visibility.Visible;
        }
                
        //Una vegada obert l'element multimedia establim la seva duració a la barra de progrés
        private void mediaElement1_MediaOpened(object sender, RoutedEventArgs e)
        {
            Posicio = mediaElement1.NaturalDuration.TimeSpan;
            pbProgres.Minimum = 0;
            pbProgres.Maximum = Posicio.TotalSeconds;
        }
                
        //Detenir la reproducció en curs (Stop)
        private void btnStop_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            btnPause.Visibility = Visibility.Hidden;
            btnPlay.Visibility = Visibility.Visible;
            mediaElement1.Stop();
        }

        //Pausem la reproducció (Pausa)
        private void btnPause_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            btnPause.Visibility = Visibility.Hidden;
            btnPlay.Visibility = Visibility.Visible;

            mediaElement1.Pause();
        }
        
        //Reproduir (Play)
        private void btnPlay_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            btnPlay.Visibility = Visibility.Hidden;
            btnPause.Visibility = Visibility.Visible;

            mediaElement1.Play();          
        }
      
        //Incrementem en una unitat el volum
        private void btnSoundUp_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            pbVolum.Value = pbVolum.Value + 1;
            mediaElement1.Volume = (double)(pbVolum.Value / 10);
        }

        //Decrementem en una unitat el volum
        private void btnSoundDown_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            pbVolum.Value = pbVolum.Value - 1;
            mediaElement1.Volume = (double)(pbVolum.Value / 10);
        }
               
        private void btnClose_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            this.Close();
        }

        private BitmapImage GetImatge(String paramIcon)
        {
            const String IconsSoundOn = @"pack://application:,,,/WPFGestioClients;component/Icons/Sound.ico";
            const String IconsSoundOff = @"pack://application:,,,/WPFGestioClients;component/Icons/SoundMute.ico";

            try
            {
                BitmapImage logo = new BitmapImage();
                logo.BeginInit();

                switch (paramIcon)
                {
                    case "On":
                        logo.UriSource = new Uri(IconsSoundOn);
                        break;
                    case "Off":
                        logo.UriSource = new Uri(IconsSoundOff);
                        break;
                }

                logo.EndInit();
                return logo;
            }
            catch (NullReferenceException ex)
            {
                MessageBox.Show(ex.Message, "VideoClub: " + ex.Source, MessageBoxButton.OK, MessageBoxImage.Error);
                return null;
            }
            catch (SystemException ex1)
            {
                MessageBox.Show(ex1.Message, "VideoClub: " + ex1.Source, MessageBoxButton.OK, MessageBoxImage.Error);
                return null;
            }
            catch (Exception ex2)
            {
                MessageBox.Show(ex2.Message, "VideoClub: " + ex2.Source, MessageBoxButton.OK, MessageBoxImage.Error);
                return null;
            }
        }

        //Activem/desactivem el volum
        private void btnSoundOnOff_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            if (mediaElement1.Volume == 0)
            {
                mediaElement1.Volume = 0.5;
                mediaElement1.IsMuted = false;
                btnSoundOnOff.Source = GetImatge("Off");
                pbVolum.Value = mediaElement1.Volume * 10;
            }else{
                mediaElement1.Volume = 0;
                mediaElement1.IsMuted = true;
                btnSoundOnOff.Source = GetImatge("On");
                pbVolum.Value = mediaElement1.Volume * 10;
            }
        }
    }
}