﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;
using System.IO;
using WPFGestioInterna.WCFVideoClubServiceReference;

namespace WPFGestioInterna
{
    /// <summary>
    /// Lógica de interacción para wndAddEditArticle.xaml
    /// </summary>
    public partial class wndAddEditArticle : Window
    {
        Service1Client client = new Service1Client();
        PELLICULA pelliculaActual;        
        PARAMETRES_APLICACIO appParam = new PARAMETRES_APLICACIO();
        String FitxerCaratulaF = "";
        String FitxerCaratulaP = "";
        String FitxerVisionatPrevi = "";
        List<GENERE> LlistaGeneresPellicula = new List<GENERE>();
        
        //Si no rep com a paràmetre una pel.ícula "nova pel.lícula", altrament "editar pel.lícula"
        public wndAddEditArticle(PELLICULA paramPellicula)
        {
            InitializeComponent();
            pelliculaActual = paramPellicula;
            appParam = client.ObtenirParametresAplicacio();

            if (pelliculaActual != null)
            {                
                CarregarDadesPellicula(paramPellicula);
                client.GuardarTemporalCopiesPellicula(paramPellicula.ID_PELLICULA); //Guardem la pel.lícula en una taula temporal per si finalment l'usuari decideix cancelar el procés (es buida automaticament el temporal)        
            }else{
                client.BuidarTemporalCopiesPellicula(); //Buidem la taula temporal de copies de pel.lícula
            }

            CarregarDataGrid();
        }

        private void CarregarDataGrid()
        {
            dtgUnitatsArticle.ItemsSource = client.ObtenirLlistatTemporalCopiesPellicula(false);

            if (dtgUnitatsArticle.Items.Count > 0)
            {
                DelUnitat.IsEnabled = true;
                EditUnitat.IsEnabled = true;
            }
            else
            {
                DelUnitat.IsEnabled = false;
                EditUnitat.IsEnabled = false;
            }
        }

        private void btnCancelar_Click(object sender, RoutedEventArgs e)
        {
            this.Close();
        }

        private void btnAcceptar_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                if (ValidarDadesIntroduides() == true) //Primer validem que les dades introduides siguin correctes
                {
                    int IdPellicula;
                    PELLICULA pelicula  = new PELLICULA();
                    int n = 0;
                    DateTime d;
                    
                    pelicula.TITOL = txtTitol.Text;
                    pelicula.DIRECTOR = txtDirector.Text;
                    pelicula.PAIS = txtPais.Text;
                    if (txtAny.Text != "" && int.TryParse(txtAny.Text, out n) != false) { pelicula.ANY = int.Parse(txtAny.Text); }
                    if (txtDataEstrena.Text != "" && DateTime.TryParse(txtDataEstrena.Text, out d) != false) { pelicula.DATA_ESTRENA = DateTime.Parse(txtDataEstrena.Text); }
                    if (txtDuracio.Text != "" && int.TryParse(txtDuracio.Text, out n) != false) { pelicula.DURACIO = int.Parse(txtDuracio.Text); }
                    pelicula.QUALIFICACIO = txtQualificacio.Text;
                    pelicula.REPARTIMENT = txtRepartiment.Text;
                    pelicula.ARGUMENT = txtArgument.Text;
                    
                    if (pelliculaActual != null) //Modificar dades de la pel.lícula
                    {
                        IdPellicula = pelliculaActual.ID_PELLICULA;
                        pelicula.ID_PELLICULA = IdPellicula;
                        client.ModificarPellicula(pelicula);
                    }else{ //Nova pel.lícula
                        IdPellicula = client.AltaPellicula(pelicula, LlistaGeneresPellicula.ToArray());
                        pelicula.ID_PELLICULA = IdPellicula;
                    }

                    List<TMP_COPIES> llitaTemporalCopies = new List<TMP_COPIES>();
                    llitaTemporalCopies = client.ObtenirLlistatTemporalCopiesPellicula(true).ToList();

                    //Guardem les noves copies o les modificacions en les existents
                    for (int i = 0; i < llitaTemporalCopies.Count; i++)
                    {
                        UNITAT_PELLICULA UnitatCopia = new UNITAT_PELLICULA();

                        UnitatCopia.ID_PELLICULA = IdPellicula;
                        UnitatCopia.LLOGUER = llitaTemporalCopies[i].LLOGUER;
                        UnitatCopia.VENDA = llitaTemporalCopies[i].VENDA;
                        UnitatCopia.PREU_VENDA = llitaTemporalCopies[i].PREU_VENDA;

                        if (llitaTemporalCopies[i].ID_UNITAT != 0)
                        {
                            UnitatCopia.ID_UNITAT = llitaTemporalCopies[i].ID_UNITAT;

                            if (llitaTemporalCopies[i].BAIXA == 1)
                            {
                                client.BaixaCopiaPellicula(llitaTemporalCopies[i].ID_UNITAT);
                            }else{
                                client.ModificarCopiaPellicula(UnitatCopia, llitaTemporalCopies[i].ID_FORMAT);
                            }
                            
                        }else{
                            client.AltaCopiaPellicula(UnitatCopia, IdPellicula, llitaTemporalCopies[i].ID_FORMAT);
                        }
                    }
                    
                    //Guardem el fitxer de la caràtula frontal (IdPellicula_F.jpg)
                    String FitxerOrigen;
                    String FitxerDestinacio;

                    /*
                     * Podem seleccionar qualsevol fitxer jpg per a les imatges o wmv per als videos, és a dir, 
                     * el nom del fitxer original no és important ja que el programari agafarà aquest fitxer i 
                     * el copiarà a la carpeta de imatges/videos amb com a nom l'identificador de la pel.lícula
                     * a la BD.
                     */

                    if (FitxerCaratulaF.Length > 0)
                    {
                        FitxerOrigen = FitxerCaratulaF;
                        FitxerDestinacio = appParam.RUTA_IMATGES + (char)92 + IdPellicula + "_F.jpg";
                        System.IO.File.Copy(FitxerOrigen, FitxerDestinacio, true);
                        pelicula.CARATULA_FRONTAL = IdPellicula + "_F.jpg";
                    }else{
                        pelicula.CARATULA_FRONTAL = txtCaratulaF.Text;
                    }

                    //Guardem el fitxer de la caràtula posterior (IdPellicula_B.jpg)
                    if (FitxerCaratulaP.Length > 0)
                    {
                        FitxerOrigen = FitxerCaratulaP;
                        FitxerDestinacio = appParam.RUTA_IMATGES + (char)92 + IdPellicula + "_B.jpg";
                        System.IO.File.Copy(FitxerOrigen, FitxerDestinacio, true);
                        pelicula.CARATULA_POSTERIOR = IdPellicula + "_B.jpg";
                    }else{
                        pelicula.CARATULA_POSTERIOR = txtCaratulaP.Text;
                    }

                    //Guardem el fitxer del video de visionat previ (IdPellicula.wmv)
                    if (FitxerVisionatPrevi.Length > 0)
                    {
                        FitxerOrigen = FitxerVisionatPrevi;
                        FitxerDestinacio = appParam.RUTA_VIDEOS + (char)92 + IdPellicula + ".wmv";
                        System.IO.File.Copy(FitxerOrigen, FitxerDestinacio, true);
                        pelicula.VISIONAT_PREVI = IdPellicula + ".wmv";
                    }else{
                        pelicula.VISIONAT_PREVI = txtVisionatPrevi.Text;
                    }

                    //Guardem els gèneres seleccionats de la pel.lícula
                    client.ModificarPellicula(pelicula);
                    client.EsborrarGeneresPellicula(IdPellicula);

                    for (int i = 1; i <= LlistaGeneresPellicula.Count; i++)
                    {
                        client.AfegirGenerePellicula(IdPellicula, LlistaGeneresPellicula[i - 1].ID_GENERE);
                    }

                    this.Close();
                }
            }
            catch (NullReferenceException ex)
            {
                MessageBox.Show(ex.Message, "VideoClub: " + ex.Source, MessageBoxButton.OK, MessageBoxImage.Error);
            }
            catch (SystemException ex1)
            {
                MessageBox.Show(ex1.Message, "VideoClub: " + ex1.Source, MessageBoxButton.OK, MessageBoxImage.Error);
            }
            catch (Exception ex2)
            {
                MessageBox.Show(ex2.Message, "VideoClub: " + ex2.Source, MessageBoxButton.OK, MessageBoxImage.Error);
            }
        }

        private void CarregarDadesPellicula(PELLICULA paramPellicula)
        {
            if (paramPellicula != null) //Modificar dades d'usuari
            {
                txtTitol.Text = paramPellicula.TITOL;
                txtDirector.Text = paramPellicula.DIRECTOR;
                txtPais.Text = paramPellicula.PAIS;
                txtAny.Text = paramPellicula.ANY.ToString();
                txtDataEstrena.Text = String.Format("{0:dd/MM/yyyy}", paramPellicula.DATA_ESTRENA);
                txtDuracio.Text = paramPellicula.DURACIO.ToString();
                
                //Recuperem els gèneres de la pel.lícula
                txtGenere.Text = client.ObtenirGeneresPellicula(paramPellicula.ID_PELLICULA);
                LlistaGeneresPellicula = client.ObtenirLlistaGeneresPellicula(paramPellicula.ID_PELLICULA).ToList();
                
                txtQualificacio.Text = paramPellicula.QUALIFICACIO;
                txtRepartiment.Text = paramPellicula.REPARTIMENT;
                txtArgument.Text = paramPellicula.ARGUMENT;

                //Recuperem les unitats (còpies) de la pel.lícula
                txtCaratulaF.Text = paramPellicula.CARATULA_FRONTAL;
                txtCaratulaP.Text = paramPellicula.CARATULA_POSTERIOR;
                txtVisionatPrevi.Text = paramPellicula.VISIONAT_PREVI;

                //Vista caràtules en miniatura                
                imgFrontal.Source = GetImatge(appParam.RUTA_IMATGES + (char)92 + paramPellicula.CARATULA_FRONTAL);
                imgPosterior.Source = GetImatge(appParam.RUTA_IMATGES + (char)92 + paramPellicula.CARATULA_POSTERIOR);
            }
        }

        private void btnCaratulaFrontal_Click(object sender, RoutedEventArgs e)
        {
            Microsoft.Win32.OpenFileDialog dlg = new Microsoft.Win32.OpenFileDialog();
            dlg.DefaultExt = ".jpg";
            dlg.Filter = "JPEG (.jpg)|*.jpg"; // Filtrar fitxers per extensió
            dlg.Multiselect = false;

            Nullable<bool> result = dlg.ShowDialog();

            if (result == true)
            {
                FitxerCaratulaF = dlg.FileName;
                txtCaratulaF.Text = FitxerCaratulaF;
                imgFrontal.Source = GetImatge(FitxerCaratulaF);
            }else{
                FitxerCaratulaF = txtCaratulaF.Text;
                imgFrontal.Source = GetImatge(FitxerCaratulaF);
            }
        }

        private void btnCartulaPosterior_Click(object sender, RoutedEventArgs e)
        {
            Microsoft.Win32.OpenFileDialog dlg = new Microsoft.Win32.OpenFileDialog();
            dlg.DefaultExt = ".jpg";
            dlg.Filter = "JPEG (.jpg)|*.jpg"; // Filtrar fitxers per extensió
            dlg.Multiselect = false;

            Nullable<bool> result = dlg.ShowDialog();

            if (result == true)
            {
                FitxerCaratulaP = dlg.FileName;
                txtCaratulaP.Text = FitxerCaratulaP;
                imgPosterior.Source = GetImatge(FitxerCaratulaP);
            }else{
                FitxerCaratulaP = txtCaratulaP.Text;                 
                imgPosterior.Source = GetImatge(FitxerCaratulaP);
            }
        }

        private void btnVisionatPrevi_Click(object sender, RoutedEventArgs e)
        {
            Microsoft.Win32.OpenFileDialog dlg = new Microsoft.Win32.OpenFileDialog();
            dlg.DefaultExt = ".wmv";
            dlg.Filter = "Archivo de audio o video de Windows Media (.wmv)|*.wmv"; // Filtrar fitxers per extensió
            dlg.Multiselect = false;

            Nullable<bool> result = dlg.ShowDialog();

            if (result == true)
            {
                FitxerVisionatPrevi = dlg.FileName;
                txtVisionatPrevi.Text = FitxerVisionatPrevi;
            }else{
                FitxerVisionatPrevi = txtVisionatPrevi.Text;
            }
        }

        private BitmapImage GetImatge(String RutaImatge)
        {
            try
            {
                const String ImgNoDisponibleFile = @"pack://application:,,,/WPFGestioInterna;component/Images/ImatgeNoDisponible.jpg";
                String CurFile = "";

                if (RutaImatge != "") { CurFile = RutaImatge; }

                if (!File.Exists(CurFile)) { CurFile = ImgNoDisponibleFile; }
                BitmapImage logo = new BitmapImage();
                logo.BeginInit();
                logo.UriSource = new Uri(CurFile);
                logo.EndInit();
                return logo;
            }
            catch (NullReferenceException ex)
            {
                MessageBox.Show(ex.Message, "VideoClub: " + ex.Source, MessageBoxButton.OK, MessageBoxImage.Error);
                return null;
            }
            catch (SystemException ex1)
            {
                MessageBox.Show(ex1.Message, "VideoClub: " + ex1.Source, MessageBoxButton.OK, MessageBoxImage.Error);
                return null;
            }
            catch (Exception ex2)
            {
                MessageBox.Show(ex2.Message, "VideoClub: " + ex2.Source, MessageBoxButton.OK, MessageBoxImage.Error);
                return null;
            }
        }

        private Boolean ValidarDadesIntroduides()
        {
            Boolean res = true;

            if (txtTitol.Text.Length == 0) 
            { 
                txtTitol.Background = Brushes.LightCoral; 
                res = false; 
            } else { 
                txtTitol.Background = Brushes.White;

                //Si donem d'alta una nova pel.lícula, comprovem que no existeixi ja a la BD
                if (pelliculaActual == null)
                {
                    if (client.ExisteixPellicula(txtTitol.Text) == true)
                    {
                        MessageBox.Show("Ya existeix en la Base de Dades una pel.lícula amb el mateix nom", this.Title, MessageBoxButton.OK, MessageBoxImage.Error);
                        txtTitol.Background = Brushes.LightCoral; 
                        res = false; 
                    }
                }
            }

            if (txtGenere.Text.Length == 0)
            {
                MessageBox.Show("No ha especificat el gènere de la pel.lícula", this.Title, MessageBoxButton.OK, MessageBoxImage.Error);
                txtGenere.Background = Brushes.LightCoral;
                res = false; 
            }else{
                txtGenere.Background = Brushes.White;
            }

            if (dtgUnitatsArticle.Items.Count <= 0)
            {
                MessageBox.Show("No ha donat d'alta cap còpia de la pel.lícula", this.Title, MessageBoxButton.OK, MessageBoxImage.Error);
                dtgUnitatsArticle.Background = Brushes.LightCoral;
                res = false;
            }else{
                dtgUnitatsArticle.Background = Brushes.White;
            }
            
            try
            {
                if (txtAny.Text.Length > 0) { int.Parse(txtAny.Text); }
                txtAny.Background = Brushes.White;
            }catch{
                txtAny.Background = Brushes.LightCoral;
                res = false;
            }

            try
            {
                if (txtDataEstrena.Text.Length > 0) { 
                    DateTime.Parse(txtDataEstrena.Text);

                    DateTime DataEstrena = DateTime.Parse(txtDataEstrena.Text);
                    if (DataEstrena > DateTime.Now.Date)
                    {
                        MessageBox.Show("La data d'estrena no pot ser superior a la data actual", this.Title, MessageBoxButton.OK, MessageBoxImage.Error);
                        txtDataEstrena.Background = Brushes.LightCoral;
                        res = false;
                    }else{
                        txtDataEstrena.Background = Brushes.White;    
                    }
                }                
            }catch{
                txtDataEstrena.Background = Brushes.LightCoral;
                res = false;
            }

            try
            {
                if (txtDuracio.Text.Length > 0) { int.Parse(txtDuracio.Text); }
                txtDuracio.Background = Brushes.White;
            }catch{
                txtDuracio.Background = Brushes.LightCoral;
                res = false;
            }

            return res;
        }

        private void Button_Click(object sender, RoutedEventArgs e)
        {
            wndSelGeneres windowGeneres;

            if (pelliculaActual != null)
            {
                windowGeneres = new wndSelGeneres(pelliculaActual.ID_PELLICULA);
                windowGeneres.ShowDialog();
            }else{
                windowGeneres = new wndSelGeneres(0);
                windowGeneres.ShowDialog();
            }
            
            if (windowGeneres.llistaGeneresPellicula != null)
            {
                LlistaGeneresPellicula = windowGeneres.llistaGeneresPellicula;
                txtGenere.Text = ObtenirGeneresPellicula(LlistaGeneresPellicula);
            }
        }

        //Mètode que retorna un String amb la descripció de tots els gèneres de la llista rebuda com a paràmetre
        private String ObtenirGeneresPellicula(List<GENERE> LlistaGeneres)
        {
            try
            {
                String StrGeneres = "";

                for (int i = 1; i <= LlistaGeneres.Count; i++)
                {
                    StrGeneres = StrGeneres + LlistaGeneres[i-1].GENERE1;
                    if (i < LlistaGeneres.Count) { StrGeneres = StrGeneres + ", "; }
                }

                return StrGeneres;
            }
            catch (NullReferenceException ex)
            {
                MessageBox.Show(ex.Message, "VideoClub: " + ex.Source, MessageBoxButton.OK, MessageBoxImage.Error);
                return "";
            }
            catch (SystemException ex1)
            {
                MessageBox.Show(ex1.Message, "VideoClub: " + ex1.Source, MessageBoxButton.OK, MessageBoxImage.Error);
                return "";
            }
            catch (Exception ex2)
            {
                MessageBox.Show(ex2.Message, "VideoClub: " + ex2.Source, MessageBoxButton.OK, MessageBoxImage.Error);
                return "";
            }
        }

        private void AddUnitat_Click(object sender, RoutedEventArgs e)
        {
            if (pelliculaActual != null)
            {
                wndAddCopia windowAddCopia = new wndAddCopia(null,pelliculaActual.ID_PELLICULA);
                windowAddCopia.ShowDialog();
            }else{
                wndAddCopia windowAddCopia = new wndAddCopia(null, 0);
                windowAddCopia.ShowDialog();
            }

            CarregarDataGrid();
        }

        private void DelUnitat_Click(object sender, RoutedEventArgs e)
        {
            TMP_COPIES SelUnitatArticle = (TMP_COPIES)dtgUnitatsArticle.SelectedItem;

            if (SelUnitatArticle != null)
            {
                String EstatCopia = client.ObtenirEstatCopiaPellicula(SelUnitatArticle.ID_UNITAT);

                if (EstatCopia == "D" || SelUnitatArticle.ID_UNITAT == 0)
                {
                    if (MessageBox.Show("Vol donar de baixa la còpia seleccionada", this.Title, MessageBoxButton.YesNo, MessageBoxImage.Question) == MessageBoxResult.Yes)
                    {
                        client.EsborrarCopiaPelliculaTemporal(SelUnitatArticle.ID_TMP);                                                
                    }
                }else{
                    MessageBox.Show("No es pot donar de baixa una còpia que esta reservada o no disponible", this.Title, MessageBoxButton.OK, MessageBoxImage.Error);
                }
            }

            CarregarDataGrid();
        }

        private void EditUnitat_Click(object sender, RoutedEventArgs e)
        {
            TMP_COPIES SelUnitatArticle = (TMP_COPIES)dtgUnitatsArticle.SelectedItem;

            if (SelUnitatArticle != null)
            {
                wndAddCopia windowAddCopia = new wndAddCopia(SelUnitatArticle, SelUnitatArticle.ID_PELLICULA);
                windowAddCopia.ShowDialog();
            }

            CarregarDataGrid();
        }
    }
}