﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;
using WPFGestioInterna.WCFVideoClubServiceReference;

namespace WPFGestioInterna
{
    /// <summary>
    /// Lógica de interacción para wndAddEditClient.xaml
    /// </summary>
    public partial class wndAddEditClient : Window
    {
        Service1Client client = new Service1Client();
        USUARI usuariActual;

        //Si el paràmetre no és null "Nou usuari", altrament "Editar usuari"
        public wndAddEditClient(USUARI paramUsuari)
        {
            InitializeComponent();
            usuariActual = paramUsuari;
            if (paramUsuari != null)
            {
                btnRestablirPsw.IsEnabled = true;
                CarregarDadesUsuari(paramUsuari);
                chkPrintCarnet.IsChecked = false;
            }
        }

        private void btnCancelar_Click(object sender, RoutedEventArgs e)
        {
            this.Close();
        }

        private void btnAcceptar_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                if (ValidarDadesIntroduides() == true)
                {
                    USUARI usuari = new USUARI();
                    String CodiUsuari = "";

                    if (txtDNI.Text != "")
                    {
                        CodiUsuari = DNIToCodiUsuari(txtDNI.Text);
                        usuari.DNI = txtDNI.Text.ToUpper();
                        usuari.CONTRASENYA = int.Parse(CodiUsuari.Substring(CodiUsuari.Length - 4));
                    }

                    usuari.NOM = txtNom.Text;
                    usuari.COGNOM1 = txtCognom1.Text;
                    usuari.COGNOM2 = txtCognom2.Text;
                    usuari.ADREÇA = txtAdreça.Text;
                    usuari.POBLACIO = txtPoblacio.Text;
                    usuari.TELEFON1 = txtTelefon1.Text;

                    //Camps opcionals: CP, Província i Telefon2
                    int n = 0;
                    if (txtCP.Text != "" && int.TryParse(txtCP.Text, out n) == false) { usuari.CP = int.Parse(txtCP.Text); }
                    if (txtProvincia.Text.Length > 0) { usuari.PROVINCIA = txtProvincia.Text; }
                    if (txtTelefon2.Text.Length > 0) { usuari.TELEFON2 = txtTelefon2.Text; }

                    if (usuariActual != null) //Modificar dades d'usuari
                    {
                        usuari.ID_USUARI = usuariActual.ID_USUARI;
                        client.ModificarUsuari(usuari);
                    }
                    else
                    { //Nou usuari  
                        txtCodiUsuari.Text = CodiUsuari;
                        txtContrasenya.Text = txtCodiUsuari.Text.Substring(txtCodiUsuari.Text.Length - 4);
                        client.AltaUsuari(usuari);
                    }

                    if (chkPrintCarnet.IsChecked == true)
                    {
                        PARAMETRES_APLICACIO AppParam = client.ObtenirParametresAplicacio();

                        wndPrintCarnet windowPrintCarnet = new wndPrintCarnet(true, AppParam.RAO_SOCIAL, usuari.ID_USUARI, usuari.NOM + ' ' + usuari.COGNOM1 + ' ' + usuari.COGNOM2, usuari.DNI);
                        windowPrintCarnet.Show();
                    }

                    this.Close();
                }
            }
            catch (NullReferenceException ex)
            {
                MessageBox.Show(ex.Message, "VideoClub: " + ex.Source, MessageBoxButton.OK, MessageBoxImage.Error);
            }
            catch (SystemException ex1)
            {
                MessageBox.Show(ex1.Message, "VideoClub: " + ex1.Source, MessageBoxButton.OK, MessageBoxImage.Error);
            }
            catch (Exception ex2)
            {
                MessageBox.Show(ex2.Message, "VideoClub: " + ex2.Source, MessageBoxButton.OK, MessageBoxImage.Error);
            }
        }

        private void CarregarDadesUsuari(USUARI paramUsuari)
        {
            if (paramUsuari != null) //Modificar dades d'usuari
            {
                txtDNI.Text = paramUsuari.DNI;
                txtNom.Text = paramUsuari.NOM;
                txtCognom1.Text = paramUsuari.COGNOM1;
                txtCognom2.Text = paramUsuari.COGNOM2;
                txtAdreça.Text = paramUsuari.ADREÇA;
                txtCP.Text = paramUsuari.CP.ToString();
                txtPoblacio.Text = paramUsuari.POBLACIO;
                txtProvincia.Text = paramUsuari.PROVINCIA;
                txtTelefon1.Text = paramUsuari.TELEFON1;
                txtTelefon2.Text = paramUsuari.TELEFON2;

                txtCodiUsuari.Text = DNIToCodiUsuari(paramUsuari.DNI.ToString());
                txtContrasenya.Text = paramUsuari.CONTRASENYA.ToString();
            }
        }

        //El DNI (menys la lletra) és igual al codi d'usuari
        private String DNIToCodiUsuari(String DNI)
        {
            String returnCodiUsuari = "";

            for (int i = 1; i <= DNI.Length; i++)
            {
                if (char.IsDigit(DNI[i - 1]))
                {
                    returnCodiUsuari = returnCodiUsuari + DNI[i - 1];
                }
                else
                {
                    break;
                }
            }

            return returnCodiUsuari;
        }

        private Boolean ValidarDadesIntroduides()
        {
            Boolean res = true;

            //Comprovem que el DNI estigui format per 8 números + 1 lletra
            if (DNIValid(txtDNI.Text) == false) { txtDNI.Background = Brushes.LightCoral; res = false; } else { txtDNI.Background = Brushes.White; }

            //Comprovem que s'haguin informat els camps obligatoris
            if (txtNom.Text.Length == 0) { txtNom.Background = Brushes.LightCoral; res = false; } else { txtNom.Background = Brushes.White; }
            if (txtCognom1.Text.Length == 0) { txtCognom1.Background = Brushes.LightCoral; res = false; } else { txtCognom1.Background = Brushes.White; }
            if (txtCognom2.Text.Length == 0) { txtCognom2.Background = Brushes.LightCoral; res = false; } else { txtCognom2.Background = Brushes.White; }
            if (txtAdreça.Text.Length == 0) { txtAdreça.Background = Brushes.LightCoral; res = false; } else { txtAdreça.Background = Brushes.White; }
            if (txtPoblacio.Text.Length == 0) { txtPoblacio.Background = Brushes.LightCoral; res = false; } else { txtPoblacio.Background = Brushes.White; }
            if (txtTelefon1.Text.Length == 0) { txtTelefon1.Background = Brushes.LightCoral; res = false; } else { txtTelefon1.Background = Brushes.White; }

            //Comprovem que el CP (si s'ha informat) sigui un valor numèric
            try
            {
                if (txtCP.Text.Length > 0)
                {
                    int.Parse(txtCP.Text);
                }
                txtCP.Background = Brushes.White;
            }
            catch
            {
                txtCP.Background = Brushes.LightCoral;
                res = false;
            }

            return res;
        }

        //El DNI ha de tenir 8 números i una lletra
        private Boolean DNIValid(String DNI)
        {
            Boolean res = true;

            if (DNI.Length < 9) { res = false; }

            if (DNI.Length > 0)
            {
                for (int i = 1; i <= DNI.Length - 1; i++)
                {
                    if (!char.IsDigit(DNI[i - 1])) { res = false; }
                }

                String LletraDNI = DNI.Substring(DNI.Length - 1);
                if (!char.IsLetter(LletraDNI[0])) { res = false; }

                if (usuariActual == null)
                {
                    if (client.ExisteixUsuari(DNI.ToUpper()) == true)
                    {
                        MessageBox.Show("Ja existeix un client registrat amb aquest DNI", this.Title, MessageBoxButton.OK, MessageBoxImage.Error);
                        res = false;
                    }
                }
            }

            return res;
        }

        //Tornem a possar la contrasenya per defecte (4 últims dígits del codi d'usuari)
        private void btnRestablirPsw_Click(object sender, RoutedEventArgs e)
        {
            txtContrasenya.Text = txtCodiUsuari.Text.Substring(txtCodiUsuari.Text.Length - 4);
        }
    }
}