﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;
using wfs = System.Windows.Forms;
using Microsoft.Win32;

using WPFGestioInterna.WCFVideoClubServiceReference;

namespace WPFGestioInterna
{
    public partial class wndMain : Window
    {
        Service1Client client = new Service1Client();
        USUARI UsuariSeleccionat;
        Boolean LoadingConfig = false;

        public wndMain()
        {
            InitializeComponent();
        }
        
        //Mètode que carrega el dataGridReserves amb les reserves que hi ha registrades a la BD
        private void CarregarDataGridReserves(USUARI paramUsuari)
        {
            try
            {
                this.Cursor = Cursors.Wait;

                if (paramUsuari != null)
                {
                    dtgReserves.ItemsSource = client.ObtenirLlistatReserves(paramUsuari.ID_USUARI);
                }
                else
                {
                    dtgReserves.ItemsSource = client.ObtenirLlistatReserves(0);
                }

                this.Cursor = Cursors.Arrow;
            }
            catch (NullReferenceException ex)
            {
                MessageBox.Show(ex.Message, "VideoClub: " + ex.Source, MessageBoxButton.OK, MessageBoxImage.Error);
            }
            catch (SystemException ex1)
            {
                MessageBox.Show(ex1.Message, "VideoClub: " + ex1.Source, MessageBoxButton.OK, MessageBoxImage.Error);
            }
            catch (Exception ex2)
            {
                MessageBox.Show(ex2.Message, "VideoClub: " + ex2.Source, MessageBoxButton.OK, MessageBoxImage.Error);
            }
        }

        //Totes les reserves actives
        private void Button_TotesReserves_Click(object sender, RoutedEventArgs e)
        {
            UsuariSeleccionat = null;
            CarregarDataGridReserves(null);
        }

        //Totes les reserves d'un client concret
        private void Button_ReservesClient_Click(object sender, RoutedEventArgs e)
        {
            wndSearchUser windowSearchUser = new wndSearchUser();
            windowSearchUser.ShowDialog();
            if (windowSearchUser.SearchedUser != null)
            {
                UsuariSeleccionat = windowSearchUser.SearchedUser;
                CarregarDataGridReserves(UsuariSeleccionat);
            }
        }

        //Acceptar reserva
        private void Button_AcceptarReserva_Click(object sender, RoutedEventArgs e)
        {
            VISTA_RESERVES_ACTIVES SelReserva = (VISTA_RESERVES_ACTIVES)dtgReserves.SelectedItem;

            if (SelReserva != null)
            {
                wndAcceptReserva windowAcceptReserva = new wndAcceptReserva(SelReserva);
                windowAcceptReserva.ShowDialog();
                CarregarDataGridReserves(null);
            }
        }

        //Cancelar reserva
        private void Button_CancelarReserva_Click(object sender, RoutedEventArgs e)
        {
            VISTA_RESERVES_ACTIVES SelReserva = (VISTA_RESERVES_ACTIVES)dtgReserves.SelectedItem;

            if (SelReserva != null)
            {
                client.CancelarReserva(SelReserva.ID_RESERVA);
                CarregarDataGridReserves(null);
            }
        }

        //Mètode que carrega el dataGridLloguers amb els lloguers que hi ha registrades a la BD
        private void CarregarDataGridLloguers(USUARI paramUsuari)
        {
            try
            {
                this.Cursor = Cursors.Wait;

                if (paramUsuari != null)
                {
                    dtgLloguers.ItemsSource = client.ObtenirLlistatLloguers(paramUsuari.ID_USUARI);
                }
                else
                {
                    dtgLloguers.ItemsSource = client.ObtenirLlistatLloguers(0);
                }

                this.Cursor = Cursors.Arrow;
            }
            catch (NullReferenceException ex)
            {
                MessageBox.Show(ex.Message, "VideoClub: " + ex.Source, MessageBoxButton.OK, MessageBoxImage.Error);
            }
            catch (SystemException ex1)
            {
                MessageBox.Show(ex1.Message, "VideoClub: " + ex1.Source, MessageBoxButton.OK, MessageBoxImage.Error);
            }
            catch (Exception ex2)
            {
                MessageBox.Show(ex2.Message, "VideoClub: " + ex2.Source, MessageBoxButton.OK, MessageBoxImage.Error);
            }
        }

        //Tots els lloguers actius
        private void Button_TotsLloguers_Click(object sender, RoutedEventArgs e)
        {
            UsuariSeleccionat = null;
            CarregarDataGridLloguers(null);
        }

        //Tots els lloguers d'un client determinat
        private void Button_LloguersClient_Click(object sender, RoutedEventArgs e)
        {
            wndSearchUser windowSearchUser = new wndSearchUser();
            windowSearchUser.ShowDialog();
            if (windowSearchUser.SearchedUser != null)
            {
                UsuariSeleccionat = windowSearchUser.SearchedUser;
                CarregarDataGridLloguers(UsuariSeleccionat);
            }
        }

        //Realitzar la devolució d'un lloguer
        private void Button_TornarLloguer_Click(object sender, RoutedEventArgs e)
        {
            VISTA_LLOGUERS_ACTIUS SelLloguer = (VISTA_LLOGUERS_ACTIUS)dtgLloguers.SelectedItem;

            if (SelLloguer != null)
            {
                wndDevolucio windowDevolucio = new wndDevolucio(SelLloguer);
                windowDevolucio.ShowDialog();
                CarregarDataGridLloguers(null);
            }
        }

        //Afegir article
        private void AddArticle_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            wndAddEditArticle windowArticle = new wndAddEditArticle(null);
            windowArticle.Show();
        }

        private void Window_Initialized(object sender, EventArgs e)
        {
            try
            {
                //Caregem els paràmetres de configuració de l'aplicaicó
                PARAMETRES_APLICACIO AppParams = client.ObtenirParametresAplicacio();

                if (AppParams == null)
                {
                    //Recuperem les credencials d'usuari de l'administrador
                    USUARI admin = client.ObtenirUsuariAdministrador();
                    if (admin != null)
                    {
                        txtNomUsuari.Text = admin.DNI;
                        txtContrasenya.Text = admin.CONTRASENYA.ToString();
                    }

                    MessageBox.Show("Abans de començar a utilitzar el programa ha d'informar els paràmetres de configuració", this.Title, MessageBoxButton.OK, MessageBoxImage.Information);
                    ItemConfiguracio.IsSelected = true;
                }
                else
                {
                    //Carreguem les dades de la primera pestanya (Reserves)
                    this.CarregarParametresAplicacio();
                    ItemReserves.IsSelected = true;
                }
            }
            catch (NullReferenceException ex)
            {
                MessageBox.Show(ex.Message, "VideoClub: " + ex.Source, MessageBoxButton.OK, MessageBoxImage.Error);
            }
            catch (SystemException ex1)
            {
                MessageBox.Show(ex1.Message, "VideoClub: " + ex1.Source, MessageBoxButton.OK, MessageBoxImage.Error);
            }
            catch (Exception ex2)
            {
                MessageBox.Show(ex2.Message, "VideoClub: " + ex2.Source, MessageBoxButton.OK, MessageBoxImage.Error);
            }
        }

        private Boolean CarregarParametresAplicacio()
        {
            try
            {
                this.Cursor = Cursors.Wait;

                PARAMETRES_APLICACIO AppParams = new PARAMETRES_APLICACIO();

                AppParams = client.ObtenirParametresAplicacio();

                if (AppParams != null)
                {
                    LoadingConfig = true;

                    //Recuperem les dades del videoclub
                    txtRaoSocial.Text = AppParams.RAO_SOCIAL;
                    txtCIF.Text = AppParams.CIF;
                    txtAdreça.Text = AppParams.ADREÇA;
                    txtTelefon.Text = AppParams.TELEFON;
                    if (AppParams.FAX != null) { txtFax.Text = AppParams.FAX; }
                    if (AppParams.EMAIL != null) { txtEmail.Text = AppParams.EMAIL; }
                    if (AppParams.WEB != null) { txtWeb.Text = AppParams.WEB; }

                    //Recuperem les credencials d'usuari de l'administrador
                    USUARI admin = client.ObtenirUsuariAdministrador();
                    if (admin != null)
                    {
                        txtNomUsuari.Text = admin.DNI;
                        txtContrasenya.Text = admin.CONTRASENYA.ToString();
                    }

                    //Recuperem els paràmetres de configuració de l'aplicació
                    txtPreuHoraLloguer.Text = AppParams.PREU_HORA_LLOGUER.ToString();
                    txtPreuMinimLloguer.Text = AppParams.PREU_MINIM_LLOGUER.ToString();
                    txtMaxHoresLloguer.Text = AppParams.MAX_HORES_LLOGUER.ToString();
                    //txtMaxArticlesLlogats.Text = AppParams.MAX_ARTICLES_LLOGATS_USUARI.ToString();
                    //txtMaxMinutsRecollida.Text = AppParams.MAX_MINUTS_RECOLLIR_RESERVA.ToString();
                    //txtMaxMinutsEnTermina.Text = AppParams.MAX_MINUTS_UTILITZACIO_TERMINAL.ToString();
                    txtRutaImatges.Text = AppParams.RUTA_IMATGES.ToString();
                    txtRutaVideos.Text = AppParams.RUTA_VIDEOS.ToString();
                    this.Cursor = Cursors.Arrow;
                    LoadingConfig = false;

                    txtStatusBar.Text = "S'han carregat els paràmetres de configuració de l'aplicació";

                    return true;
                }
                else
                {
                    this.Cursor = Cursors.Arrow;
                    return false;
                }
            }
            catch (NullReferenceException ex)
            {
                MessageBox.Show(ex.Message, "VideoClub: " + ex.Source, MessageBoxButton.OK, MessageBoxImage.Error);
                return false;
            }
            catch (SystemException ex1)
            {
                MessageBox.Show(ex1.Message, "VideoClub: " + ex1.Source, MessageBoxButton.OK, MessageBoxImage.Error);
                return false;
            }
            catch (Exception ex2)
            {
                MessageBox.Show(ex2.Message, "VideoClub: " + ex2.Source, MessageBoxButton.OK, MessageBoxImage.Error);
                return false;
            }
        }

        private void CarregarDataGridUsuaris(int TipusConsulta)
        {
            try
            {
                this.Cursor = Cursors.Wait;

                //TipusConsulta: 0-Tots els usuaris, 1-Usuaris actius i 2-Usuaris inactius
                dtgClients.ItemsSource = client.ObtenirUsuaris(TipusConsulta);

                this.Cursor = Cursors.Arrow;
            }
            catch (NullReferenceException ex)
            {
                MessageBox.Show(ex.Message, "VideoClub: " + ex.Source, MessageBoxButton.OK, MessageBoxImage.Error);
            }
            catch (SystemException ex1)
            {
                MessageBox.Show(ex1.Message, "VideoClub: " + ex1.Source, MessageBoxButton.OK, MessageBoxImage.Error);
            }
            catch (Exception ex2)
            {
                MessageBox.Show(ex2.Message, "VideoClub: " + ex2.Source, MessageBoxButton.OK, MessageBoxImage.Error);
            }
        }

        private void btnTotsClients_Click(object sender, RoutedEventArgs e)
        {
            this.CarregarDataGridUsuaris(0);
        }

        private void btnClientsActius_Click(object sender, RoutedEventArgs e)
        {
            this.CarregarDataGridUsuaris(1);
        }

        private void btnCliensInactius_Click(object sender, RoutedEventArgs e)
        {
            this.CarregarDataGridUsuaris(2);
        }

        void CarregarDataGridArticles(int TipusConsulta)
        {
            try
            {
                this.Cursor = Cursors.Wait;

                switch (TipusConsulta)
                {
                    case 0: //Tots els articles actius
                        dtgArticles.ItemsSource = client.ObtenirLlistatTotsArticles();
                        dtgArticles.Columns[5].Visibility = Visibility.Visible;
                        dtgArticles.Columns[6].Visibility = Visibility.Visible;
                        break;
                    case 1: //Tots els articles de lloguer actius
                        dtgArticles.ItemsSource = client.ObtenirLlistatArticlesLloguer();
                        dtgArticles.Columns[5].Visibility = Visibility.Hidden;
                        dtgArticles.Columns[6].Visibility = Visibility.Hidden;
                        break;
                    case 2: //Tots els articles de venda actius
                        dtgArticles.ItemsSource = client.ObtenirLlistatArticlesVenda();
                        dtgArticles.Columns[5].Visibility = Visibility.Hidden;
                        dtgArticles.Columns[6].Visibility = Visibility.Hidden;
                        break;
                }

                this.Cursor = Cursors.Arrow;
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }

        private void btnTotsArticles_Click(object sender, RoutedEventArgs e)
        {
            this.CarregarDataGridArticles(0);
        }

        private void btnArticlesLloguer_Click(object sender, RoutedEventArgs e)
        {
            this.CarregarDataGridArticles(1);
        }

        private void btnArticlesVenda_Click(object sender, RoutedEventArgs e)
        {
            this.CarregarDataGridArticles(2);
        }

        private void CarregarDataGridFormats(int TipusConsulta)
        {
            try
            {
                this.Cursor = Cursors.Wait;

                //TipusConsulta: 0-Tots els formats, 1-Formats actius i 2-Formats inactius
                dtgFormats.ItemsSource = client.ObtenirFormats(TipusConsulta);

                this.Cursor = Cursors.Arrow;
            }
            catch (NullReferenceException ex)
            {
                MessageBox.Show(ex.Message, "VideoClub: " + ex.Source, MessageBoxButton.OK, MessageBoxImage.Error);
            }
            catch (SystemException ex1)
            {
                MessageBox.Show(ex1.Message, "VideoClub: " + ex1.Source, MessageBoxButton.OK, MessageBoxImage.Error);
            }
            catch (Exception ex2)
            {
                MessageBox.Show(ex2.Message, "VideoClub: " + ex2.Source, MessageBoxButton.OK, MessageBoxImage.Error);
            }
        }

        private void CarregarDataGridInforme1()
        {
            try
            {
                this.Cursor = Cursors.Wait;

                dtgInforme1.ItemsSource = client.ObtenirLlistaLloguersForaTerminiDevolucio();

                this.Cursor = Cursors.Arrow;
            }
            catch (NullReferenceException ex)
            {
                MessageBox.Show(ex.Message, "VideoClub: " + ex.Source, MessageBoxButton.OK, MessageBoxImage.Error);
            }
            catch (SystemException ex1)
            {
                MessageBox.Show(ex1.Message, "VideoClub: " + ex1.Source, MessageBoxButton.OK, MessageBoxImage.Error);
            }
            catch (Exception ex2)
            {
                MessageBox.Show(ex2.Message, "VideoClub: " + ex2.Source, MessageBoxButton.OK, MessageBoxImage.Error);
            }
        }

        private void CarregarDataGridInforme2()
        {
            try
            {
                this.Cursor = Cursors.Wait;

                dtgInforme2.ItemsSource = client.ObtenirLlistaRendimentEconomicArticles();

                this.Cursor = Cursors.Arrow;
            }
            catch (NullReferenceException ex)
            {
                MessageBox.Show(ex.Message, "VideoClub: " + ex.Source, MessageBoxButton.OK, MessageBoxImage.Error);
            }
            catch (SystemException ex1)
            {
                MessageBox.Show(ex1.Message, "VideoClub: " + ex1.Source, MessageBoxButton.OK, MessageBoxImage.Error);
            }
            catch (Exception ex2)
            {
                MessageBox.Show(ex2.Message, "VideoClub: " + ex2.Source, MessageBoxButton.OK, MessageBoxImage.Error);
            }
        }

        private void CarregarDataGridInforme3()
        {
            try
            {
                this.Cursor = Cursors.Wait;

                dtgInforme3.ItemsSource = client.ObtenirLlistaNovetats();

                this.Cursor = Cursors.Arrow;
            }
            catch (NullReferenceException ex)
            {
                MessageBox.Show(ex.Message, "VideoClub: " + ex.Source, MessageBoxButton.OK, MessageBoxImage.Error);
            }
            catch (SystemException ex1)
            {
                MessageBox.Show(ex1.Message, "VideoClub: " + ex1.Source, MessageBoxButton.OK, MessageBoxImage.Error);
            }
            catch (Exception ex2)
            {
                MessageBox.Show(ex2.Message, "VideoClub: " + ex2.Source, MessageBoxButton.OK, MessageBoxImage.Error);
            }
        }

        private void btnTotsFormats_Click(object sender, RoutedEventArgs e)
        {
            CarregarDataGridFormats(0);
        }

        private void btnFormatsActius_Click(object sender, RoutedEventArgs e)
        {
            CarregarDataGridFormats(1);
        }

        private void btnFormatsInactius_Click(object sender, RoutedEventArgs e)
        {
            CarregarDataGridFormats(2);
        }

        private void CarregarDataGridGeneres(int TipusConsulta)
        {
            try
            {
                this.Cursor = Cursors.Wait;

                //TipusConsulta: 0-Tots els gèneres, 1-Gèneres actius i 2-Gèneres inactius
                dtgGeneres.ItemsSource = client.ObtenirGeneres(TipusConsulta);

                this.Cursor = Cursors.Arrow;
            }
            catch (NullReferenceException ex)
            {
                MessageBox.Show(ex.Message, "VideoClub: " + ex.Source, MessageBoxButton.OK, MessageBoxImage.Error);
            }
            catch (SystemException ex1)
            {
                MessageBox.Show(ex1.Message, "VideoClub: " + ex1.Source, MessageBoxButton.OK, MessageBoxImage.Error);
            }
            catch (Exception ex2)
            {
                MessageBox.Show(ex2.Message, "VideoClub: " + ex2.Source, MessageBoxButton.OK, MessageBoxImage.Error);
            }
        }

        private void btnTotsGeneres_Click(object sender, RoutedEventArgs e)
        {
            CarregarDataGridGeneres(0);
        }

        private void btnGeneresActius_Click(object sender, RoutedEventArgs e)
        {
            CarregarDataGridGeneres(1);
        }

        private void btnGeneresInactius_Click(object sender, RoutedEventArgs e)
        {
            CarregarDataGridGeneres(2);
        }

        private void btnAddClient_Click(object sender, RoutedEventArgs e)
        {
            wndAddEditClient windowClient = new wndAddEditClient(null);
            windowClient.ShowDialog();
            this.CarregarDataGridUsuaris(1);
        }

        private void btnDelClient_Click(object sender, RoutedEventArgs e)
        {
            USUARI SelClient = (USUARI)dtgClients.SelectedItem;

            if (SelClient != null)
            {
                if (MessageBox.Show("Vol donar de baixa el client seleccionat (" + SelClient.NOM + ' ' + SelClient.COGNOM1 + ' ' + SelClient.COGNOM2 + ")", this.Title, MessageBoxButton.YesNo, MessageBoxImage.Question) == MessageBoxResult.Yes)
                {
                    client.BaixaUsuari(SelClient.ID_USUARI);
                    this.CarregarDataGridUsuaris(1);
                }
            }
        }

        private void btnEditClient_Click(object sender, RoutedEventArgs e)
        {
            USUARI SelClient = (USUARI)dtgClients.SelectedItem;

            if (SelClient != null)
            {
                wndAddEditClient windowClient = new wndAddEditClient(SelClient);
                windowClient.ShowDialog();
                this.CarregarDataGridUsuaris(1);
            }
        }

        private void btnPrintClientCard_Click(object sender, RoutedEventArgs e)
        {
            USUARI SelClient = (USUARI)dtgClients.SelectedItem;

            if (SelClient != null)
            {
                wndPrintCarnet windowPrintCarnet = new wndPrintCarnet(true, txtRaoSocial.Text, SelClient.ID_USUARI, SelClient.NOM + ' ' + SelClient.COGNOM1 + ' ' + SelClient.COGNOM2, SelClient.DNI);
                windowPrintCarnet.Show();
            }
        }

        private void btnConfigAcceptar_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                if (ValidarDadesConfiguració() == true)
                {
                    PARAMETRES_APLICACIO newParams = new PARAMETRES_APLICACIO();

                    newParams.RAO_SOCIAL = txtRaoSocial.Text;
                    newParams.CIF = txtCIF.Text;
                    newParams.ADREÇA = txtAdreça.Text;
                    newParams.EMAIL = txtEmail.Text;
                    newParams.TELEFON = txtTelefon.Text;
                    newParams.FAX = txtFax.Text;
                    newParams.WEB = txtWeb.Text;
                    newParams.RUTA_IMATGES = txtRutaImatges.Text;
                    newParams.RUTA_VIDEOS = txtRutaVideos.Text;

                    int n = 0;
                    //if (int.TryParse(txtMaxArticlesLlogats.Text, out n) == true) { newParams.MAX_ARTICLES_LLOGATS_USUARI = int.Parse(txtMaxArticlesLlogats.Text); } else { newParams.MAX_ARTICLES_LLOGATS_USUARI = 0; }
                    if (int.TryParse(txtMaxHoresLloguer.Text, out n) == true) { newParams.MAX_HORES_LLOGUER = int.Parse(txtMaxHoresLloguer.Text); } else { newParams.MAX_HORES_LLOGUER = 0; }
                    //if (int.TryParse(txtMaxMinutsRecollida.Text, out n) == true) { newParams.MAX_MINUTS_RECOLLIR_RESERVA = int.Parse(txtMaxMinutsRecollida.Text); } else { newParams.MAX_MINUTS_RECOLLIR_RESERVA = 0; }
                    //if (int.TryParse(txtMaxMinutsEnTermina.Text, out n) == true) { newParams.MAX_MINUTS_UTILITZACIO_TERMINAL = int.Parse(txtMaxMinutsEnTermina.Text); } else { newParams.MAX_MINUTS_UTILITZACIO_TERMINAL = 0; }

                    Double d = 0;
                    if (Double.TryParse(txtPreuHoraLloguer.Text, out d) == true) { newParams.PREU_HORA_LLOGUER = Double.Parse(txtPreuHoraLloguer.Text); } else { newParams.PREU_HORA_LLOGUER = 0; }
                    if (Double.TryParse(txtPreuMinimLloguer.Text, out d) == true) { newParams.PREU_MINIM_LLOGUER = Double.Parse(txtPreuMinimLloguer.Text); } else { newParams.PREU_MINIM_LLOGUER = 0; }

                    //Guardem els possibles canvis realitzats en els paràmetres de l'aplicació
                    client.GuardarConfiguracio(newParams);
                    //Guardem els possibles canvis realitzats en al contrasenya de l'administrador
                    client.CanviaContrasenyaAdministrador(int.Parse(txtContrasenya.Text));

                    btnConfigAcceptar.IsEnabled = false;
                    btnConfigCancelar.IsEnabled = false;
                    CarregarParametresAplicacio();
                }
            }
            catch (NullReferenceException ex)
            {
                MessageBox.Show(ex.Message, "VideoClub: " + ex.Source, MessageBoxButton.OK, MessageBoxImage.Error);
            }
            catch (SystemException ex1)
            {
                MessageBox.Show(ex1.Message, "VideoClub: " + ex1.Source, MessageBoxButton.OK, MessageBoxImage.Error);
            }
            catch (Exception ex2)
            {
                MessageBox.Show(ex2.Message, "VideoClub: " + ex2.Source, MessageBoxButton.OK, MessageBoxImage.Error);
            }
        }

        private void btnRutaImages_Click(object sender, RoutedEventArgs e)
        {
            var folderBrowser = new wfs.FolderBrowserDialog();

            if (folderBrowser.ShowDialog() == wfs.DialogResult.OK)
            {
                txtRutaImatges.Text = folderBrowser.SelectedPath;
            }
        }

        private void btnRutaVideos_Click(object sender, RoutedEventArgs e)
        {
            var folderBrowser = new wfs.FolderBrowserDialog();

            if (folderBrowser.ShowDialog() == wfs.DialogResult.OK)
            {
                txtRutaVideos.Text = folderBrowser.SelectedPath;
            }
        }

        private Boolean ValidarDadesConfiguració()
        {
            Boolean res = true;

            //Comprovem que s'haguin informat tots els camps
            if (txtRaoSocial.Text.Length == 0) { txtRaoSocial.Background = Brushes.LightCoral; res = false; } else { txtRaoSocial.Background = Brushes.White; }
            if (txtCIF.Text.Length == 0) { txtCIF.Background = Brushes.LightCoral; res = false; } else { txtCIF.Background = Brushes.White; }
            if (txtAdreça.Text.Length == 0) { txtAdreça.Background = Brushes.LightCoral; res = false; } else { txtAdreça.Background = Brushes.White; }
            if (txtTelefon.Text.Length == 0) { txtTelefon.Background = Brushes.LightCoral; res = false; } else { txtTelefon.Background = Brushes.White; }
            if (txtContrasenya.Text.Length == 0) { txtContrasenya.Background = Brushes.LightCoral; res = false; } else { txtContrasenya.Background = Brushes.White; }

            int n = 0;
            //if (int.TryParse(txtMaxArticlesLlogats.Text, out n) != false) { txtMaxArticlesLlogats.Background = Brushes.White; } else { txtMaxArticlesLlogats.Background = Brushes.LightCoral; res = false; }
            if (int.TryParse(txtMaxHoresLloguer.Text, out n) != false) { txtMaxHoresLloguer.Background = Brushes.White; } else { txtMaxHoresLloguer.Background = Brushes.LightCoral; res = false; }
            //if (int.TryParse(txtMaxMinutsRecollida.Text, out n) != false) { txtMaxMinutsRecollida.Background = Brushes.White; } else { txtMaxMinutsRecollida.Background = Brushes.LightCoral; res = false; }
            //if (int.TryParse(txtMaxMinutsEnTermina.Text, out n) != false) { txtMaxMinutsEnTermina.Background = Brushes.White; } else { txtMaxMinutsEnTermina.Background = Brushes.LightCoral; res = false; }

            Double d = 0;
            if (Double.TryParse(txtPreuHoraLloguer.Text, out d) != false) { txtPreuHoraLloguer.Background = Brushes.White; } else { txtPreuHoraLloguer.Background = Brushes.LightCoral; res = false; }
            if (Double.TryParse(txtPreuMinimLloguer.Text, out d) != false) { txtPreuMinimLloguer.Background = Brushes.White; } else { txtPreuMinimLloguer.Background = Brushes.LightCoral; res = false; }

            return res;
        }

        private void btnConfigCancelar_Click(object sender, RoutedEventArgs e)
        {
            btnConfigAcceptar.IsEnabled = false;
            btnConfigCancelar.IsEnabled = false;
            CarregarParametresAplicacio();
        }

        private void Change_Config_Param(object sender, TextChangedEventArgs e)
        {
            if (LoadingConfig == false)
            {
                btnConfigAcceptar.IsEnabled = true;
                btnConfigCancelar.IsEnabled = true;
            }
        }

        private void btnEditArticle_Click(object sender, RoutedEventArgs e)
        {
            PELLICULA SelPellicula = null;

            if (dtgArticles.SelectedItem == null) { return; }

            switch (dtgArticles.SelectedItem.GetType().ToString())
            {
                case "WPFGestioInterna.WCFVideoClubServiceReference.VISTA_ARTICLES_ACTIUS_LLOGUER":
                    VISTA_ARTICLES_ACTIUS_LLOGUER SelArticle = (VISTA_ARTICLES_ACTIUS_LLOGUER)dtgArticles.SelectedItem;
                    SelPellicula = client.ObtenirPellicula(SelArticle.ID_PELLICULA);
                    break;
                case "WPFGestioInterna.WCFVideoClubServiceReference.VISTA_ARTICLES_ACTIUS_VENDA":
                    VISTA_ARTICLES_ACTIUS_VENDA SelArticle1 = (VISTA_ARTICLES_ACTIUS_VENDA)dtgArticles.SelectedItem;
                    SelPellicula = client.ObtenirPellicula(SelArticle1.ID_PELLICULA);
                    break;
                case "WPFGestioInterna.WCFVideoClubServiceReference.VISTA_ARTICLES_ACTIUS":
                    VISTA_ARTICLES_ACTIUS SelArticle2 = (VISTA_ARTICLES_ACTIUS)dtgArticles.SelectedItem;
                    SelPellicula = client.ObtenirPellicula(SelArticle2.ID_PELLICULA);
                    break;
            }

            if (SelPellicula != null)
            {
                wndAddEditArticle windowArticle = new wndAddEditArticle(SelPellicula);
                windowArticle.ShowDialog();
                this.CarregarDataGridArticles(1);
            }
        }

        private void btnDelArticle_Click(object sender, RoutedEventArgs e)
        {
            PELLICULA SelPellicula = null;

            if (dtgArticles.SelectedItem == null) { return; }

            switch (dtgArticles.SelectedItem.GetType().ToString())
            {
                case "WPFGestioInterna.WCFVideoClubServiceReference.VISTA_ARTICLES_ACTIUS_LLOGUER":
                    VISTA_ARTICLES_ACTIUS_LLOGUER SelArticle = (VISTA_ARTICLES_ACTIUS_LLOGUER)dtgArticles.SelectedItem;
                    SelPellicula = client.ObtenirPellicula(SelArticle.ID_PELLICULA);
                    break;
                case "WPFGestioInterna.WCFVideoClubServiceReference.VISTA_ARTICLES_ACTIUS_VENDA":
                    VISTA_ARTICLES_ACTIUS_VENDA SelArticle1 = (VISTA_ARTICLES_ACTIUS_VENDA)dtgArticles.SelectedItem;
                    SelPellicula = client.ObtenirPellicula(SelArticle1.ID_PELLICULA);
                    break;
                case "WPFGestioInterna.WCFVideoClubServiceReference.VISTA_ARTICLES_ACTIUS":
                    VISTA_ARTICLES_ACTIUS SelArticle2 = (VISTA_ARTICLES_ACTIUS)dtgArticles.SelectedItem;
                    SelPellicula = client.ObtenirPellicula(SelArticle2.ID_PELLICULA);
                    break;
            }

            if (SelPellicula != null)
            {
                if (MessageBox.Show("Vol donar de baixa l'article seleccionat (" + SelPellicula.TITOL + "). Seliminarà l'article seleccionat i totes les seves còpies.", this.Title, MessageBoxButton.YesNo, MessageBoxImage.Question) == MessageBoxResult.Yes)
                {
                    client.BaixaPellicula(SelPellicula.ID_PELLICULA);
                    this.CarregarDataGridArticles(1);
                }
            }
        }

        private void btnAddArticle_Click(object sender, RoutedEventArgs e)
        {
            wndAddEditArticle windowArticle = new wndAddEditArticle(null);
            windowArticle.ShowDialog();
            this.CarregarDataGridArticles(1);
        }

        private void btnAddFormat_Click(object sender, RoutedEventArgs e)
        {
            wndAddFormat windowAddFormat = new wndAddFormat();
            windowAddFormat.ShowDialog();
            this.CarregarDataGridFormats(1);
        }

        private void btnDelFormat_Click(object sender, RoutedEventArgs e)
        {
            FORMAT SelFormat = (FORMAT)dtgFormats.SelectedItem;

            if (SelFormat != null)
            {
                if (MessageBox.Show("Vol donar de baixa el format seleccionat (" + SelFormat.FORMAT1 + ")", this.Title, MessageBoxButton.YesNo, MessageBoxImage.Question) == MessageBoxResult.Yes)
                {
                    client.BaixaFormat(SelFormat.ID_FORMAT);
                    this.CarregarDataGridFormats(1);
                }
            }
        }

        private void btnAddGenere_Click(object sender, RoutedEventArgs e)
        {
            wndAddGenere windowAddGenere = new wndAddGenere();
            windowAddGenere.ShowDialog();
            this.CarregarDataGridFormats(1);
        }

        private void btnDelGenere_Click(object sender, RoutedEventArgs e)
        {
            GENERE SelGenere = (GENERE)dtgGeneres.SelectedItem;

            if (SelGenere != null)
            {
                if (MessageBox.Show("Vol donar de baixa el gènere seleccionat (" + SelGenere.GENERE1 + ")", this.Title, MessageBoxButton.YesNo, MessageBoxImage.Question) == MessageBoxResult.Yes)
                {
                    client.BaixaGenere(SelGenere.ID_GENERE);
                    this.CarregarDataGridGeneres(1);
                }
            }
        }

        private void btnActualitzarInforme1_Click(object sender, RoutedEventArgs e)
        {
            CarregarDataGridInforme1();
        }

        private void btnActualitzarInforme2_Click(object sender, RoutedEventArgs e)
        {
            CarregarDataGridInforme2();
        }

        private void btnActualitzarInforme3_Click(object sender, RoutedEventArgs e)
        {
            CarregarDataGridInforme3();
        }

        private void btnImprimirInforme1_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                PrintDialog printDialog = new PrintDialog();

                if (printDialog.ShowDialog() == true)
                {
                    printDialog.PrintVisual(dtgInforme1, "PrintInforme_" + DateTime.Now.ToString("dd/mm/yyyy"));
                }
            }
            catch
            {
                MessageBox.Show("Si us plau, comprovi que tingui instal.lada alguna impresora", this.Title, MessageBoxButton.OK, MessageBoxImage.Error);
            }
        }

        private void btnImprimirInforme2_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                PrintDialog printDialog = new PrintDialog();

                if (printDialog.ShowDialog() == true)
                {
                    printDialog.PrintVisual(dtgInforme2, "PrintInforme_" + DateTime.Now.ToString("dd/mm/yyyy"));
                }
            }
            catch
            {
                MessageBox.Show("Si us plau, comprovi que tingui instal.lada alguna impresora", this.Title, MessageBoxButton.OK, MessageBoxImage.Error);
            }
        }

        private void btnImprimirInforme3_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                PrintDialog printDialog = new PrintDialog();

                if (printDialog.ShowDialog() == true)
                {
                    printDialog.PrintVisual(dtgInforme3, "PrintInforme_" + DateTime.Now.ToString("dd/mm/yyyy"));
                }
            }
            catch
            {
                MessageBox.Show("Si us plau, comprovi que tingui instal.lada alguna impresora", this.Title, MessageBoxButton.OK, MessageBoxImage.Error);
            }
        }

        private void dtgInforme1_Loaded(object sender, RoutedEventArgs e)
        {
            if (tabControl1.SelectedIndex == 6)
            {
                if (dtgInforme1.Items.Count > 0) { dtgInforme1.SelectedIndex = 0; }
                txtStatusBar.Text = "S'han trobat " + dtgInforme1.Items.Count + " articles llogats que estan fora de termini de devolució";
            }
        }

        private void dtgInforme2_Loaded(object sender, RoutedEventArgs e)
        {
            if (tabControl1.SelectedIndex == 7)
            {
                if (dtgInforme2.Items.Count > 0) { dtgInforme2.SelectedIndex = 0; }
                txtStatusBar.Text = "S'han trobat " + dtgInforme2.Items.Count + " articles";
            }
        }

        private void dtgReserves_Loaded(object sender, RoutedEventArgs e)
        {
            if (tabControl1.SelectedIndex == 0)
            {
                if (dtgReserves.Items.Count > 0) { dtgReserves.SelectedIndex = 0; }
                txtStatusBar.Text = "S'han trobat " + dtgReserves.Items.Count + " pel.lícules reservades";
                if (UsuariSeleccionat != null) { txtStatusBar.Text = txtStatusBar.Text + " per l'usuari " + UsuariSeleccionat.NOM + ' ' + UsuariSeleccionat.COGNOM1 + ' ' + UsuariSeleccionat.COGNOM2; }
            }
        }

        private void dtgLloguers_Loaded(object sender, RoutedEventArgs e)
        {
            if (tabControl1.SelectedIndex == 1)
            {
                if (dtgLloguers.Items.Count > 0) { dtgLloguers.SelectedIndex = 0; }
                txtStatusBar.Text = "S'han trobat " + dtgLloguers.Items.Count + " pel.lícules llogades";
                if (UsuariSeleccionat != null) { txtStatusBar.Text = txtStatusBar.Text + " per l'usuari " + UsuariSeleccionat.NOM + ' ' + UsuariSeleccionat.COGNOM1 + ' ' + UsuariSeleccionat.COGNOM2; }
            }
        }

        private void dtgClients_Loaded(object sender, RoutedEventArgs e)
        {
            if (tabControl1.SelectedIndex == 2)
            {
                if (dtgClients.Items.Count > 0) { dtgClients.SelectedIndex = 0; }
                txtStatusBar.Text = "S'han trobat " + dtgClients.Items.Count + " clients";
            }
        }

        private void dtgArticles_Loaded(object sender, RoutedEventArgs e)
        {
            if (tabControl1.SelectedIndex == 3)
            {
                if (dtgArticles.Items.Count > 0) { dtgArticles.SelectedIndex = 0; }
                txtStatusBar.Text = "S'han trobat " + dtgArticles.Items.Count + " articles";
            }
        }

        private void dtgFormats_Loaded(object sender, RoutedEventArgs e)
        {
            if (tabControl1.SelectedIndex == 4)
            {
                if (dtgFormats.Items.Count > 0) { dtgFormats.SelectedIndex = 0; }
                txtStatusBar.Text = "S'han trobat " + dtgFormats.Items.Count + " formats";
            }
        }

        private void dtgGeneres_Loaded(object sender, RoutedEventArgs e)
        {
            if (tabControl1.SelectedIndex == 5)
            {
                if (dtgGeneres.Items.Count > 0) { dtgGeneres.SelectedIndex = 0; }
                txtStatusBar.Text = "S'han trobat " + dtgGeneres.Items.Count + " gèneres";
            }
        }

        private void dtgInforme3_Loaded(object sender, RoutedEventArgs e)
        {
            if (tabControl1.SelectedIndex == 8)
            {
                if (dtgInforme3.Items.Count > 0) { dtgInforme3.SelectedIndex = 0; }
                txtStatusBar.Text = "S'han trobat " + dtgInforme3.Items.Count + " novetats";
            }
        }

        private void TabControl_OnSelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            if (e.OriginalSource != this.tabControl1)
            {
                return;
            }else{
                if (tabControl1.SelectedIndex >= 0)
                {
                    if (tabControl1.SelectedIndex != 0) { tbReserves.Visibility = Visibility.Hidden; }
                    if (tabControl1.SelectedIndex != 1) { tbLloguers.Visibility = Visibility.Hidden; }
                    if (tabControl1.SelectedIndex != 2) { tbClients.Visibility = Visibility.Hidden; }
                    if (tabControl1.SelectedIndex != 3) { tbArticles.Visibility = Visibility.Hidden; }
                    if (tabControl1.SelectedIndex != 4) { tbFormats.Visibility = Visibility.Hidden; }
                    if (tabControl1.SelectedIndex != 5) { tbGeneres.Visibility = Visibility.Hidden; }
                    if (tabControl1.SelectedIndex != 6) { tbInforme1.Visibility = Visibility.Hidden; }
                    if (tabControl1.SelectedIndex != 7) { tbInforme2.Visibility = Visibility.Hidden; }
                    if (tabControl1.SelectedIndex != 8) { tbInforme3.Visibility = Visibility.Hidden; }

                    if (tabControl1.SelectedIndex != 9 && (btnConfigAcceptar.IsEnabled == true || btnConfigCancelar.IsEnabled == true))
                    {
                        MessageBox.Show("Abans d’abandonar la pestanya de configuració ha de validar o descartar els canvis realitzats", this.Title, MessageBoxButton.OK, MessageBoxImage.Exclamation);
                        ItemConfiguracio.IsSelected = true;
                        tbConfiguracio.Visibility = Visibility.Visible;
                    }
                    else
                    {
                        if (tabControl1.SelectedIndex != 9) { tbConfiguracio.Visibility = Visibility.Hidden; }

                        switch (tabControl1.SelectedIndex)
                        {
                            case 0: //Reserves                                              
                                tbReserves.Visibility = Visibility.Visible;
                                this.CarregarDataGridReserves(null);
                                break;
                            case 1: //Lloguers                    
                                tbLloguers.Visibility = Visibility.Visible;
                                this.CarregarDataGridLloguers(null);
                                break;
                            case 2: //Clients                    
                                tbClients.Visibility = Visibility.Visible;
                                this.CarregarDataGridUsuaris(1);
                                break;
                            case 3: //Articles
                                tbArticles.Visibility = Visibility.Visible;
                                this.CarregarDataGridArticles(1);
                                break;
                            case 4: //Formats
                                tbFormats.Visibility = Visibility.Visible;
                                this.CarregarDataGridFormats(1);
                                break;
                            case 5: //Gèneres
                                tbGeneres.Visibility = Visibility.Visible;
                                this.CarregarDataGridGeneres(1);
                                break;
                            case 6: //Informe1:  
                                tbInforme1.Visibility = Visibility.Visible;
                                this.CarregarDataGridInforme1();                            
                                break;
                            case 7: //Informe2:                    
                                tbInforme2.Visibility = Visibility.Visible;
                                this.CarregarDataGridInforme2();
                                break;
                            case 8: //Informe3:                    
                                tbInforme3.Visibility = Visibility.Visible;
                                this.CarregarDataGridInforme3();
                                break;
                            case 9: //Configuració                    
                                tbConfiguracio.Visibility = Visibility.Visible;
                                this.CarregarParametresAplicacio();
                                break;
                            default:
                                break;
                        }
                    }
                }
            }
        }
    }
}