﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;

using WPFGestioInterna.WCFVideoClubServiceReference;

namespace WPFGestioInterna
{
    /// <summary>
    /// Lógica de interacción para wndSelGeneres.xaml
    /// </summary>
    public partial class wndSelGeneres : Window
    {
        int IdPelliculaActual;
        List<GENERE> llistaGeneresDisponibles = new List<GENERE>();
        public List<GENERE> llistaGeneresPellicula = new List<GENERE>();

        Service1Client client = new Service1Client();

        public wndSelGeneres(int paramIdPellicula)
        {
            IdPelliculaActual = paramIdPellicula;
            InitializeComponent();            
        }

        private void btnAcceptar_Click(object sender, RoutedEventArgs e)
        {
            if (dtgGeneresPelicula.Items.Count <= 0)
            {
                MessageBox.Show("Ha de seleccionar almenys 1 gènere",this.Title,MessageBoxButton.OK,MessageBoxImage.Error);
            }else{
                this.Close();
            }
        }

        private void btnCancelar_Click(object sender, RoutedEventArgs e)
        {            
            llistaGeneresPellicula = null;
            this.Close();
        }

        // Mètode que carrega el dataGridGèneres amb els gèneres que hi ha registrats a la BD
        private void CarregarDataGridGeneres()
        {
            try
            {   
                //Omplim el grid amb tots els gèneres disponibles
                llistaGeneresDisponibles = client.ObtenirLlistaGeneresNOPellicula(IdPelliculaActual).ToList();
                dtgGeneres.Items.Clear();
                dtgGeneres.ItemsSource = llistaGeneresDisponibles;
                if (dtgGeneres.Items.Count > 0) { dtgGeneres.SelectedIndex = 0; }

                //Omplim el grid amb els gèneres actius de la pellicula en qüestió
                llistaGeneresPellicula = client.ObtenirLlistaGeneresPellicula(IdPelliculaActual).ToList();
                dtgGeneresPelicula.Items.Clear();
                dtgGeneresPelicula.ItemsSource = llistaGeneresPellicula;
                if (dtgGeneresPelicula.Items.Count > 0) { dtgGeneresPelicula.SelectedIndex = 0; }

                if (dtgGeneres.Items.Count > 0) { btnAddGenere.IsEnabled = true; } else { btnAddGenere.IsEnabled = false; }
                if (dtgGeneresPelicula.Items.Count > 0) { btnDelGenere.IsEnabled = true; } else { btnDelGenere.IsEnabled = false; }
            }
            catch (NullReferenceException ex)
            {
                MessageBox.Show(ex.Message, "VideoClub: " + ex.Source, MessageBoxButton.OK, MessageBoxImage.Error);
            }
            catch (SystemException ex1)
            {
                MessageBox.Show(ex1.Message, "VideoClub: " + ex1.Source, MessageBoxButton.OK, MessageBoxImage.Error);
            }
            catch (Exception ex2)
            {
                MessageBox.Show(ex2.Message, "VideoClub: " + ex2.Source, MessageBoxButton.OK, MessageBoxImage.Error);
            }
        }

        private void Window_Initialized(object sender, EventArgs e)
        {
            try
            {
                CarregarDataGridGeneres();
            }
            catch (NullReferenceException ex)
            {
                MessageBox.Show(ex.Message, "VideoClub: " + ex.Source, MessageBoxButton.OK, MessageBoxImage.Error);
            }
            catch (SystemException ex1)
            {
                MessageBox.Show(ex1.Message, "VideoClub: " + ex1.Source, MessageBoxButton.OK, MessageBoxImage.Error);
            }
            catch (Exception ex2)
            {
                MessageBox.Show(ex2.Message, "VideoClub: " + ex2.Source, MessageBoxButton.OK, MessageBoxImage.Error);
            }
        }

        private void btnAddGenere_Click(object sender, RoutedEventArgs e)
        {
            AddGenere();
        }

        private void btnDelGenere_Click(object sender, RoutedEventArgs e)
        {
            DelGenere();
        }

        private void dtgGeneres_MouseDoubleClick(object sender, MouseButtonEventArgs e)
        {
            AddGenere();
        }

        private void dtgGeneresPelicula_MouseDoubleClick(object sender, MouseButtonEventArgs e)
        {
            DelGenere();
        }

        private void AddGenere()
        {
            GENERE SelGenere = (GENERE)dtgGeneres.SelectedItem;

            if (SelGenere != null)
            {
                //Afegim el gènere a la llista de gèneres de la pel.lícula
                llistaGeneresPellicula.Add(SelGenere);
                //Eliminem el gènere de la llista de gèneres disponibles
                llistaGeneresDisponibles.Remove(SelGenere);

                dtgGeneres.ItemsSource = null;
                dtgGeneres.Items.Clear();                
                dtgGeneres.ItemsSource = llistaGeneresDisponibles;                
                if (dtgGeneres.Items.Count > 0) { dtgGeneres.SelectedIndex = 0; }

                dtgGeneresPelicula.ItemsSource = null;
                dtgGeneresPelicula.Items.Clear();                
                dtgGeneresPelicula.ItemsSource = llistaGeneresPellicula;
                if (dtgGeneresPelicula.Items.Count > 0) { dtgGeneresPelicula.SelectedIndex = 0; }
            }

            if (dtgGeneres.Items.Count > 0) { btnAddGenere.IsEnabled = true; } else { btnAddGenere.IsEnabled = false; }
            if (dtgGeneresPelicula.Items.Count > 0) { btnDelGenere.IsEnabled = true; } else { btnDelGenere.IsEnabled = false; }
        }

        private void DelGenere()
        {
            GENERE SelGenere = (GENERE)dtgGeneresPelicula.SelectedItem;

            if (SelGenere != null)
            {
                //Eliminem el gènere de la llista de gèneres de la pel.lícula
                llistaGeneresPellicula.Remove(SelGenere);
                //Afegim el gènere a la llista de gèneres disponibles
                llistaGeneresDisponibles.Add(SelGenere);

                dtgGeneres.ItemsSource = null;
                dtgGeneres.Items.Clear();
                dtgGeneres.ItemsSource = llistaGeneresDisponibles;
                if (dtgGeneres.Items.Count > 0) { dtgGeneres.SelectedIndex = 0; }

                dtgGeneresPelicula.ItemsSource = null;
                dtgGeneresPelicula.Items.Clear();
                dtgGeneresPelicula.ItemsSource = llistaGeneresPellicula;
                if (dtgGeneresPelicula.Items.Count > 0) { dtgGeneresPelicula.SelectedIndex = 0; }
            }

            if (dtgGeneres.Items.Count > 0) { btnAddGenere.IsEnabled = true; } else { btnAddGenere.IsEnabled = false; }
            if (dtgGeneresPelicula.Items.Count > 0) { btnDelGenere.IsEnabled = true; } else { btnDelGenere.IsEnabled = false; }
        }
    }
}